
//
//  Pmd.cpp - Packed Multiplication/Division
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 31 Dec 2001 - James Leiterman
//
//  Last Modification
//
//
//  stab=4
//



#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>

    //  Book Development Library Modules
    
#include "Pmd.h"
#include "PmdEx.h"



    //  Multiplication Highs & lows
vmp_pmulhBProc   vmp_pmulhB;
vmp_pmulhBProc  vmp_pmulhB0;
vmp_pmulhuBProc vmp_pmulhuB;
vmp_pmulhuBProc vmp_pmulhuB0;
vmp_pmulluBProc vmp_pmulluB;
vmp_pmulluBProc vmp_pmulluB0;

vmp_pmulhHProc  vmp_pmulhH;
vmp_pmulhHProc  vmp_pmulhH0;
vmp_pmulhuHProc vmp_pmulhuH;
vmp_pmulhuHProc vmp_pmulhuH0;
vmp_pmulluHProc vmp_pmulluH;
vmp_pmulluHProc vmp_pmulluH0;

vmp_pmulluWProc vmp_pmulluW;
vmp_pmulluWProc vmp_pmulluW0;

    //  Multiplication Hi/Lo

vmp_pmulhlBProc  vmp_pmulhlB;
vmp_pmulhlBProc  vmp_pmulhlB0;
vmp_pmulhluBProc vmp_pmulhluB;
vmp_pmulhluBProc vmp_pmulhluB0;
vmp_pmulhlHProc  vmp_pmulhlH;
vmp_pmulhlHProc  vmp_pmulhlH0;
vmp_pmulhluHProc vmp_pmulhluH;
vmp_pmulhluHProc vmp_pmulhluH0;

    //  Multiplication Odd/Even

vmp_pmullueBProc vmp_pmullueB;
vmp_pmullueBProc vmp_pmullueB0;
vmp_pmulluoBProc vmp_pmulluoB;
vmp_pmulluoBProc vmp_pmulluoB0;
vmp_pmulleBProc  vmp_pmulleB;
vmp_pmulleBProc  vmp_pmulleB0;
vmp_pmulloBProc  vmp_pmulloB;
vmp_pmulloBProc  vmp_pmulloB0;

vmp_pmullueHProc vmp_pmullueH;
vmp_pmullueHProc vmp_pmullueH0;
vmp_pmulluoHProc vmp_pmulluoH;
vmp_pmulluoHProc vmp_pmulluoH0;
vmp_pmulleHProc  vmp_pmulleH;
vmp_pmulleHProc  vmp_pmulleH0;
vmp_pmulloHProc  vmp_pmulloH;
vmp_pmulloHProc  vmp_pmulloH0;



//
//  Packed signed Multiplication High(8bit) = 16x8bit
//
void vmp_pmulhBGeneric( int8 * const pbD, const int8 * const pbA, const int8 * const pbB )
{
    ASSERT_PTR4(pbD);
    ASSERT_PTR4(pbA);
    ASSERT_PTR4(pbB);

    *(pbD+0)  = (int8)((((int)*(pbA+0))  * ((int)*(pbB+0)))  >> 8);
    *(pbD+1)  = (int8)((((int)*(pbA+1))  * ((int)*(pbB+1)))  >> 8);
    *(pbD+2)  = (int8)((((int)*(pbA+2))  * ((int)*(pbB+2)))  >> 8);
    *(pbD+3)  = (int8)((((int)*(pbA+3))  * ((int)*(pbB+3)))  >> 8);
    *(pbD+4)  = (int8)((((int)*(pbA+4))  * ((int)*(pbB+4)))  >> 8);
    *(pbD+5)  = (int8)((((int)*(pbA+5))  * ((int)*(pbB+5)))  >> 8);
    *(pbD+6)  = (int8)((((int)*(pbA+6))  * ((int)*(pbB+6)))  >> 8);
    *(pbD+7)  = (int8)((((int)*(pbA+7))  * ((int)*(pbB+7)))  >> 8);
    *(pbD+8)  = (int8)((((int)*(pbA+8))  * ((int)*(pbB+8)))  >> 8);
    *(pbD+9)  = (int8)((((int)*(pbA+9))  * ((int)*(pbB+9)))  >> 8);
    *(pbD+10) = (int8)((((int)*(pbA+10)) * ((int)*(pbB+10))) >> 8);
    *(pbD+11) = (int8)((((int)*(pbA+11)) * ((int)*(pbB+11))) >> 8);
    *(pbD+12) = (int8)((((int)*(pbA+12)) * ((int)*(pbB+12))) >> 8);
    *(pbD+13) = (int8)((((int)*(pbA+13)) * ((int)*(pbB+13))) >> 8);
    *(pbD+14) = (int8)((((int)*(pbA+14)) * ((int)*(pbB+14))) >> 8);
    *(pbD+15) = (int8)((((int)*(pbA+15)) * ((int)*(pbB+15))) >> 8);
}                                   



//
//  Packed unsigned Multiplication High(8bit) = 16x8bit
//
void vmp_pmulhuBGeneric( uint8 * const pbD, const uint8 * const pbA, const uint8 * const pbB )
{
    ASSERT_PTR4(pbD);
    ASSERT_PTR4(pbA);
    ASSERT_PTR4(pbB);

    *(pbD+0)  = (uint8)((((int)*(pbA+0))  * ((uint)*(pbB+0)))  >> 8);
    *(pbD+1)  = (uint8)((((int)*(pbA+1))  * ((uint)*(pbB+1)))  >> 8);
    *(pbD+2)  = (uint8)((((int)*(pbA+2))  * ((uint)*(pbB+2)))  >> 8);
    *(pbD+3)  = (uint8)((((int)*(pbA+3))  * ((uint)*(pbB+3)))  >> 8);
    *(pbD+4)  = (uint8)((((int)*(pbA+4))  * ((uint)*(pbB+4)))  >> 8);
    *(pbD+5)  = (uint8)((((int)*(pbA+5))  * ((uint)*(pbB+5)))  >> 8);
    *(pbD+6)  = (uint8)((((int)*(pbA+6))  * ((uint)*(pbB+6)))  >> 8);
    *(pbD+7)  = (uint8)((((int)*(pbA+7))  * ((uint)*(pbB+7)))  >> 8);
    *(pbD+8)  = (uint8)((((int)*(pbA+8))  * ((uint)*(pbB+8)))  >> 8);
    *(pbD+9)  = (uint8)((((int)*(pbA+9))  * ((uint)*(pbB+9)))  >> 8);
    *(pbD+10) = (uint8)((((int)*(pbA+10)) * ((uint)*(pbB+10))) >> 8);
    *(pbD+11) = (uint8)((((int)*(pbA+11)) * ((uint)*(pbB+11))) >> 8);
    *(pbD+12) = (uint8)((((int)*(pbA+12)) * ((uint)*(pbB+12))) >> 8);
    *(pbD+13) = (uint8)((((int)*(pbA+13)) * ((uint)*(pbB+13))) >> 8);
    *(pbD+14) = (uint8)((((int)*(pbA+14)) * ((uint)*(pbB+14))) >> 8);
    *(pbD+15) = (uint8)((((int)*(pbA+15)) * ((uint)*(pbB+15))) >> 8);
}                                   



//
//  Packed (un)signed Multiplication Low(8bit) = 16x8bit
//
void vmp_pmulluBGeneric( uint8 * const pbD, const uint8 * const pbA, const uint8 * const pbB )
{
    ASSERT_PTR4(pbD);
    ASSERT_PTR4(pbA);
    ASSERT_PTR4(pbB);
    *(pbD+0)  = (uint8)((((uint)*(pbA+0))  * ((uint)*(pbB+0)))  & 0x00ff);
    *(pbD+1)  = (uint8)((((uint)*(pbA+1))  * ((uint)*(pbB+1)))  & 0x00ff);
    *(pbD+2)  = (uint8)((((uint)*(pbA+2))  * ((uint)*(pbB+2)))  & 0x00ff);
    *(pbD+3)  = (uint8)((((uint)*(pbA+3))  * ((uint)*(pbB+3)))  & 0x00ff);
    *(pbD+4)  = (uint8)((((uint)*(pbA+4))  * ((uint)*(pbB+4)))  & 0x00ff);
    *(pbD+5)  = (uint8)((((uint)*(pbA+5))  * ((uint)*(pbB+5)))  & 0x00ff);
    *(pbD+6)  = (uint8)((((uint)*(pbA+6))  * ((uint)*(pbB+6)))  & 0x00ff);
    *(pbD+7)  = (uint8)((((uint)*(pbA+7))  * ((uint)*(pbB+7)))  & 0x00ff);
    *(pbD+8)  = (uint8)((((uint)*(pbA+8))  * ((uint)*(pbB+8)))  & 0x00ff);
    *(pbD+9)  = (uint8)((((uint)*(pbA+9))  * ((uint)*(pbB+9)))  & 0x00ff);
    *(pbD+10) = (uint8)((((uint)*(pbA+10)) * ((uint)*(pbB+10))) & 0x00ff);
    *(pbD+11) = (uint8)((((uint)*(pbA+11)) * ((uint)*(pbB+11))) & 0x00ff);
    *(pbD+12) = (uint8)((((uint)*(pbA+12)) * ((uint)*(pbB+12))) & 0x00ff);
    *(pbD+13) = (uint8)((((uint)*(pbA+13)) * ((uint)*(pbB+13))) & 0x00ff);
    *(pbD+14) = (uint8)((((uint)*(pbA+14)) * ((uint)*(pbB+14))) & 0x00ff);
    *(pbD+15) = (uint8)((((uint)*(pbA+15)) * ((uint)*(pbB+15))) & 0x00ff);
}                                   



//
//  Packed signed Multiplication High(16bit) = 8x16bit
//
void vmp_pmulhHGeneric( int16 * const phD, const int16 * const phA, const int16 * const phB )
{
    ASSERT_PTR4(phD);
    ASSERT_PTR4(phA);
    ASSERT_PTR4(phB);

    *(phD+0) = (int16)((((int)*(phA+0)) * ((int)*(phB+0))) >> 16);
    *(phD+1) = (int16)((((int)*(phA+1)) * ((int)*(phB+1))) >> 16);
    *(phD+2) = (int16)((((int)*(phA+2)) * ((int)*(phB+2))) >> 16);
    *(phD+3) = (int16)((((int)*(phA+3)) * ((int)*(phB+3))) >> 16);
    *(phD+4) = (int16)((((int)*(phA+4)) * ((int)*(phB+4))) >> 16);
    *(phD+5) = (int16)((((int)*(phA+5)) * ((int)*(phB+5))) >> 16);
    *(phD+6) = (int16)((((int)*(phA+6)) * ((int)*(phB+6))) >> 16);
    *(phD+7) = (int16)((((int)*(phA+7)) * ((int)*(phB+7))) >> 16);
}                                   



//
//  Packed unsigned Multiplication High(16bit) = 8x16bit
//
void vmp_pmulhuHGeneric( uint16 * const phD, const uint16 * const phA, const uint16 * const phB )
{
    ASSERT_PTR4(phD);
    ASSERT_PTR4(phA);
    ASSERT_PTR4(phB);

    *(phD+0) = (uint16)((((uint)*(phA+0)) * ((uint)*(phB+0))) >> 16);
    *(phD+1) = (uint16)((((uint)*(phA+1)) * ((uint)*(phB+1))) >> 16);
    *(phD+2) = (uint16)((((uint)*(phA+2)) * ((uint)*(phB+2))) >> 16);
    *(phD+3) = (uint16)((((uint)*(phA+3)) * ((uint)*(phB+3))) >> 16);
    *(phD+4) = (uint16)((((uint)*(phA+4)) * ((uint)*(phB+4))) >> 16);
    *(phD+5) = (uint16)((((uint)*(phA+5)) * ((uint)*(phB+5))) >> 16);
    *(phD+6) = (uint16)((((uint)*(phA+6)) * ((uint)*(phB+6))) >> 16);
    *(phD+7) = (uint16)((((uint)*(phA+7)) * ((uint)*(phB+7))) >> 16);
}                                   



//
//  Packed (un)signed Multiplication Low(16bit) = 8x16bit
//
void vmp_pmulluHGeneric( uint16 * const phD, const uint16 * const phA, const uint16 * const phB )
{
    ASSERT_PTR4(phD);
    ASSERT_PTR4(phA);
    ASSERT_PTR4(phB);
    *(phD+0) = (uint16)((((uint)*(phA+0)) * ((uint)*(phB+0))) & 0x0000ffff);
    *(phD+1) = (uint16)((((uint)*(phA+1)) * ((uint)*(phB+1))) & 0x0000ffff);
    *(phD+2) = (uint16)((((uint)*(phA+2)) * ((uint)*(phB+2))) & 0x0000ffff);
    *(phD+3) = (uint16)((((uint)*(phA+3)) * ((uint)*(phB+3))) & 0x0000ffff);
    *(phD+4) = (uint16)((((uint)*(phA+4)) * ((uint)*(phB+4))) & 0x0000ffff);
    *(phD+5) = (uint16)((((uint)*(phA+5)) * ((uint)*(phB+5))) & 0x0000ffff);
    *(phD+6) = (uint16)((((uint)*(phA+6)) * ((uint)*(phB+6))) & 0x0000ffff);
    *(phD+7) = (uint16)((((uint)*(phA+7)) * ((uint)*(phB+7))) & 0x0000ffff);
}                                   



//
//  Packed (un)signed Multiplication Low(32bit) = 4x32bit
//
void vmp_pmulluWGeneric( uint32 * const pwD, const uint32 * const pwA, const uint32 * const pwB )
{
    ASSERT_PTR4(pwD);
    ASSERT_PTR4(pwA);
    ASSERT_PTR4(pwB);

    *(pwD+0) = *(pwA+0) * *(pwB+0);
    *(pwD+1) = *(pwA+1) * *(pwB+1);
    *(pwD+2) = *(pwA+2) * *(pwB+2);
    *(pwD+3) = *(pwA+3) * *(pwB+3);
}                                   



//______________________________________________________



//
//  Packed signed Multiplication Hi/Lo(2x8bit) = 16x8bit
//
void vmp_pmulhlBGeneric( int8 * const pbHi, int8 * const pbLo, const int8 * const pbA, const int8 * const pbB )
{
    int hilo[16];

    ASSERT_PTR4(pbHi);
    ASSERT_PTR4(pbLo);
    ASSERT_PTR4(pbA);
    ASSERT_PTR4(pbB);

    hilo[0]  = ((int)*(pbA+0))  * ((int)*(pbB+0));
    hilo[1]  = ((int)*(pbA+1))  * ((int)*(pbB+1));
    hilo[2]  = ((int)*(pbA+2))  * ((int)*(pbB+2));
    hilo[3]  = ((int)*(pbA+3))  * ((int)*(pbB+3));
    hilo[4]  = ((int)*(pbA+4))  * ((int)*(pbB+4));
    hilo[5]  = ((int)*(pbA+5))  * ((int)*(pbB+5));
    hilo[6]  = ((int)*(pbA+6))  * ((int)*(pbB+6));
    hilo[7]  = ((int)*(pbA+7))  * ((int)*(pbB+7));
    hilo[8]  = ((int)*(pbA+8))  * ((int)*(pbB+8));
    hilo[9]  = ((int)*(pbA+9))  * ((int)*(pbB+9));
    hilo[10] = ((int)*(pbA+10)) * ((int)*(pbB+10));
    hilo[11] = ((int)*(pbA+11)) * ((int)*(pbB+11));
    hilo[12] = ((int)*(pbA+12)) * ((int)*(pbB+12));
    hilo[13] = ((int)*(pbA+13)) * ((int)*(pbB+13));
    hilo[14] = ((int)*(pbA+14)) * ((int)*(pbB+14));
    hilo[15] = ((int)*(pbA+15)) * ((int)*(pbB+15));

    *(pbHi+0)  = (int8)(hilo[0]  >> 8);
    *(pbHi+1)  = (int8)(hilo[1]  >> 8);
    *(pbHi+2)  = (int8)(hilo[2]  >> 8);
    *(pbHi+3)  = (int8)(hilo[3]  >> 8);
    *(pbHi+4)  = (int8)(hilo[4]  >> 8);
    *(pbHi+5)  = (int8)(hilo[5]  >> 8);
    *(pbHi+6)  = (int8)(hilo[6]  >> 8);
    *(pbHi+7)  = (int8)(hilo[7]  >> 8);
    *(pbHi+8)  = (int8)(hilo[8]  >> 8);
    *(pbHi+9)  = (int8)(hilo[9]  >> 8);
    *(pbHi+10) = (int8)(hilo[10] >> 8);
    *(pbHi+11) = (int8)(hilo[11] >> 8);
    *(pbHi+12) = (int8)(hilo[12] >> 8);
    *(pbHi+13) = (int8)(hilo[13] >> 8);
    *(pbHi+14) = (int8)(hilo[14] >> 8);
    *(pbHi+15) = (int8)(hilo[15] >> 8);

    *(pbLo+0)  = (int8)(hilo[0]  & 0x00ff);
    *(pbLo+1)  = (int8)(hilo[1]  & 0x00ff);
    *(pbLo+2)  = (int8)(hilo[2]  & 0x00ff);
    *(pbLo+3)  = (int8)(hilo[3]  & 0x00ff);
    *(pbLo+4)  = (int8)(hilo[4]  & 0x00ff);
    *(pbLo+5)  = (int8)(hilo[5]  & 0x00ff);
    *(pbLo+6)  = (int8)(hilo[6]  & 0x00ff);
    *(pbLo+7)  = (int8)(hilo[7]  & 0x00ff);
    *(pbLo+8)  = (int8)(hilo[8]  & 0x00ff);
    *(pbLo+9)  = (int8)(hilo[9]  & 0x00ff);
    *(pbLo+10) = (int8)(hilo[10] & 0x00ff);
    *(pbLo+11) = (int8)(hilo[11] & 0x00ff);
    *(pbLo+12) = (int8)(hilo[12] & 0x00ff);
    *(pbLo+13) = (int8)(hilo[13] & 0x00ff);
    *(pbLo+14) = (int8)(hilo[14] & 0x00ff);
    *(pbLo+15) = (int8)(hilo[15] & 0x00ff);
}                                   



//
//  Packed unsigned Multiplication Hi/Lo(2x8bit) = 16x8bit
//
void vmp_pmulhluBGeneric( uint8 * const pbHi, uint8 * const pbLo, const uint8 * const pbA, const uint8 * const pbB )
{
    uint hilo[16];

    ASSERT_PTR4(pbHi);
    ASSERT_PTR4(pbLo);
    ASSERT_PTR4(pbA);
    ASSERT_PTR4(pbB);

    hilo[0]  = ((uint)*(pbA+0))  * ((uint)*(pbB+0));
    hilo[1]  = ((uint)*(pbA+1))  * ((uint)*(pbB+1));
    hilo[2]  = ((uint)*(pbA+2))  * ((uint)*(pbB+2));
    hilo[3]  = ((uint)*(pbA+3))  * ((uint)*(pbB+3));
    hilo[4]  = ((uint)*(pbA+4))  * ((uint)*(pbB+4));
    hilo[5]  = ((uint)*(pbA+5))  * ((uint)*(pbB+5));
    hilo[6]  = ((uint)*(pbA+6))  * ((uint)*(pbB+6));
    hilo[7]  = ((uint)*(pbA+7))  * ((uint)*(pbB+7));
    hilo[8]  = ((uint)*(pbA+8))  * ((uint)*(pbB+8));
    hilo[9]  = ((uint)*(pbA+9))  * ((uint)*(pbB+9));
    hilo[10] = ((uint)*(pbA+10)) * ((uint)*(pbB+10));
    hilo[11] = ((uint)*(pbA+11)) * ((uint)*(pbB+11));
    hilo[12] = ((uint)*(pbA+12)) * ((uint)*(pbB+12));
    hilo[13] = ((uint)*(pbA+13)) * ((uint)*(pbB+13));
    hilo[14] = ((uint)*(pbA+14)) * ((uint)*(pbB+14));
    hilo[15] = ((uint)*(pbA+15)) * ((uint)*(pbB+15));

    *(pbHi+0)  = (uint8)(hilo[0]  >> 8);
    *(pbHi+1)  = (uint8)(hilo[1]  >> 8);
    *(pbHi+2)  = (uint8)(hilo[2]  >> 8);
    *(pbHi+3)  = (uint8)(hilo[3]  >> 8);
    *(pbHi+4)  = (uint8)(hilo[4]  >> 8);
    *(pbHi+5)  = (uint8)(hilo[5]  >> 8);
    *(pbHi+6)  = (uint8)(hilo[6]  >> 8);
    *(pbHi+7)  = (uint8)(hilo[7]  >> 8);
    *(pbHi+8)  = (uint8)(hilo[8]  >> 8);
    *(pbHi+9)  = (uint8)(hilo[9]  >> 8);
    *(pbHi+10) = (uint8)(hilo[10] >> 8);
    *(pbHi+11) = (uint8)(hilo[11] >> 8);
    *(pbHi+12) = (uint8)(hilo[12] >> 8);
    *(pbHi+13) = (uint8)(hilo[13] >> 8);
    *(pbHi+14) = (uint8)(hilo[14] >> 8);
    *(pbHi+15) = (uint8)(hilo[15] >> 8);

    *(pbLo+0)  = (uint8)(hilo[0]  & 0x00ff);
    *(pbLo+1)  = (uint8)(hilo[1]  & 0x00ff);
    *(pbLo+2)  = (uint8)(hilo[2]  & 0x00ff);
    *(pbLo+3)  = (uint8)(hilo[3]  & 0x00ff);
    *(pbLo+4)  = (uint8)(hilo[4]  & 0x00ff);
    *(pbLo+5)  = (uint8)(hilo[5]  & 0x00ff);
    *(pbLo+6)  = (uint8)(hilo[6]  & 0x00ff);
    *(pbLo+7)  = (uint8)(hilo[7]  & 0x00ff);
    *(pbLo+8)  = (uint8)(hilo[8]  & 0x00ff);
    *(pbLo+9)  = (uint8)(hilo[9]  & 0x00ff);
    *(pbLo+10) = (uint8)(hilo[10] & 0x00ff);
    *(pbLo+11) = (uint8)(hilo[11] & 0x00ff);
    *(pbLo+12) = (uint8)(hilo[12] & 0x00ff);
    *(pbLo+13) = (uint8)(hilo[13] & 0x00ff);
    *(pbLo+14) = (uint8)(hilo[14] & 0x00ff);
    *(pbLo+15) = (uint8)(hilo[15] & 0x00ff);
}                                   



//
//  Packed signed Multiplication Hi/Lo(2x16bit) = 8x16bit
//
void vmp_pmulhlHGeneric( int16 * const phHi, int16 * const phLo, const int16 * const phA, const int16 * const phB )
{
    int hilo[8];

    ASSERT_PTR4(phHi);
    ASSERT_PTR4(phLo);
    ASSERT_PTR4(phA);
    ASSERT_PTR4(phB);

    hilo[0] = ((int)*(phA+0)) * ((int)*(phB+0));
    hilo[1] = ((int)*(phA+1)) * ((int)*(phB+1));
    hilo[2] = ((int)*(phA+2)) * ((int)*(phB+2));
    hilo[3] = ((int)*(phA+3)) * ((int)*(phB+3));
    hilo[4] = ((int)*(phA+4)) * ((int)*(phB+4));
    hilo[5] = ((int)*(phA+5)) * ((int)*(phB+5));
    hilo[6] = ((int)*(phA+6)) * ((int)*(phB+6));
    hilo[7] = ((int)*(phA+7)) * ((int)*(phB+7));

    *(phHi+0) = (int16)(hilo[0] >> 16);
    *(phHi+1) = (int16)(hilo[1] >> 16);
    *(phHi+2) = (int16)(hilo[2] >> 16);
    *(phHi+3) = (int16)(hilo[3] >> 16);
    *(phHi+4) = (int16)(hilo[4] >> 16);
    *(phHi+5) = (int16)(hilo[5] >> 16);
    *(phHi+6) = (int16)(hilo[6] >> 16);
    *(phHi+7) = (int16)(hilo[7] >> 16);

    *(phLo+0) = (int16)(hilo[0] & 0x0000ffff);
    *(phLo+1) = (int16)(hilo[1] & 0x0000ffff);
    *(phLo+2) = (int16)(hilo[2] & 0x0000ffff);
    *(phLo+3) = (int16)(hilo[3] & 0x0000ffff);
    *(phLo+4) = (int16)(hilo[4] & 0x0000ffff);
    *(phLo+5) = (int16)(hilo[5] & 0x0000ffff);
    *(phLo+6) = (int16)(hilo[6] & 0x0000ffff);
    *(phLo+7) = (int16)(hilo[7] & 0x0000ffff);
}                                   



//
//  Packed unsigned Multiplication Hi/Lo(2x16bit) = 8x16bit
//
void vmp_pmulhluHGeneric( uint16 * const phHi, uint16 * const phLo, const uint16 * const phA, const uint16 * const phB )
{
    uint hilo[8];

    ASSERT_PTR4(phHi);
    ASSERT_PTR4(phLo);
    ASSERT_PTR4(phA);
    ASSERT_PTR4(phB);

    hilo[0] = ((uint)*(phA+0)) * ((uint)*(phB+0));
    hilo[1] = ((uint)*(phA+1)) * ((uint)*(phB+1));
    hilo[2] = ((uint)*(phA+2)) * ((uint)*(phB+2));
    hilo[3] = ((uint)*(phA+3)) * ((uint)*(phB+3));
    hilo[4] = ((uint)*(phA+4)) * ((uint)*(phB+4));
    hilo[5] = ((uint)*(phA+5)) * ((uint)*(phB+5));
    hilo[6] = ((uint)*(phA+6)) * ((uint)*(phB+6));
    hilo[7] = ((uint)*(phA+7)) * ((uint)*(phB+7));

    *(phHi+0) = (uint16)(hilo[0] >> 16);
    *(phHi+1) = (uint16)(hilo[1] >> 16);
    *(phHi+2) = (uint16)(hilo[2] >> 16);
    *(phHi+3) = (uint16)(hilo[3] >> 16);
    *(phHi+4) = (uint16)(hilo[4] >> 16);
    *(phHi+5) = (uint16)(hilo[5] >> 16);
    *(phHi+6) = (uint16)(hilo[6] >> 16);
    *(phHi+7) = (uint16)(hilo[7] >> 16);

    *(phLo+0) = (uint16)(hilo[0] & 0x0000ffff);
    *(phLo+1) = (uint16)(hilo[1] & 0x0000ffff);
    *(phLo+2) = (uint16)(hilo[2] & 0x0000ffff);
    *(phLo+3) = (uint16)(hilo[3] & 0x0000ffff);
    *(phLo+4) = (uint16)(hilo[4] & 0x0000ffff);
    *(phLo+5) = (uint16)(hilo[5] & 0x0000ffff);
    *(phLo+6) = (uint16)(hilo[6] & 0x0000ffff);
    *(phLo+7) = (uint16)(hilo[7] & 0x0000ffff);
}                                   



//______________________________________________________


//
//  Packed unsigned Multiplication 16bit Even = 16x8bit
//
void vmp_pmullueBGeneric( uint16 * const phD, const uint8 * const pbA, const uint8 * const pbB )
{
    ASSERT_PTR4(phD);
    ASSERT_PTR4(pbA);
    ASSERT_PTR4(pbB);

    *(phD+0) = ((uint)*(pbA+0))  * ((uint)*(pbB+0));
    *(phD+1) = ((uint)*(pbA+2))  * ((uint)*(pbB+2));
    *(phD+2) = ((uint)*(pbA+4))  * ((uint)*(pbB+4));
    *(phD+3) = ((uint)*(pbA+6))  * ((uint)*(pbB+6));
    *(phD+4) = ((uint)*(pbA+8))  * ((uint)*(pbB+8));
    *(phD+5) = ((uint)*(pbA+10)) * ((uint)*(pbB+10));
    *(phD+6) = ((uint)*(pbA+12)) * ((uint)*(pbB+12));
    *(phD+7) = ((uint)*(pbA+14)) * ((uint)*(pbB+14));
}                                   



//
//  Packed unsigned Multiplication 16bit Odd = 16x8bit
//
void vmp_pmulluoBGeneric( uint16 * const phD, const uint8 * const pbA, const uint8 * const pbB )
{
    ASSERT_PTR4(phD);
    ASSERT_PTR4(pbA);
    ASSERT_PTR4(pbB);

    *(phD+0) = ((uint)*(pbA+1))  * ((uint)*(pbB+1));
    *(phD+1) = ((uint)*(pbA+3))  * ((uint)*(pbB+3));
    *(phD+2) = ((uint)*(pbA+5))  * ((uint)*(pbB+5));
    *(phD+3) = ((uint)*(pbA+7))  * ((uint)*(pbB+7));
    *(phD+4) = ((uint)*(pbA+9))  * ((uint)*(pbB+9));
    *(phD+5) = ((uint)*(pbA+11)) * ((uint)*(pbB+11));
    *(phD+6) = ((uint)*(pbA+13)) * ((uint)*(pbB+13));
    *(phD+7) = ((uint)*(pbA+15)) * ((uint)*(pbB+15));
}                                   



//
//  Packed signed Multiplication 16bit Even = 16x8bit
//
void vmp_pmulleBGeneric( int16 * const phD, const int8 * const pbA, const int8 * const pbB )
{
    ASSERT_PTR4(phD);
    ASSERT_PTR4(pbA);
    ASSERT_PTR4(pbB);

    *(phD+0) = ((int)*(pbA+0))  * ((int)*(pbB+0));
    *(phD+1) = ((int)*(pbA+2))  * ((int)*(pbB+2));
    *(phD+2) = ((int)*(pbA+4))  * ((int)*(pbB+4));
    *(phD+3) = ((int)*(pbA+6))  * ((int)*(pbB+6));
    *(phD+4) = ((int)*(pbA+8))  * ((int)*(pbB+8));
    *(phD+5) = ((int)*(pbA+10)) * ((int)*(pbB+10));
    *(phD+6) = ((int)*(pbA+12)) * ((int)*(pbB+12));
    *(phD+7) = ((int)*(pbA+14)) * ((int)*(pbB+14));
}                                   



//
//  Packed signed Multiplication 16bit Odd = 16x8bit
//
void vmp_pmulloBGeneric( int16 * const phD, const int8 * const pbA, const int8 * const pbB )
{
    ASSERT_PTR4(phD);
    ASSERT_PTR4(pbA);
    ASSERT_PTR4(pbB);

    *(phD+0) = ((int)*(pbA+1))  * ((int)*(pbB+1));
    *(phD+1) = ((int)*(pbA+3))  * ((int)*(pbB+3));
    *(phD+2) = ((int)*(pbA+5))  * ((int)*(pbB+5));
    *(phD+3) = ((int)*(pbA+7))  * ((int)*(pbB+7));
    *(phD+4) = ((int)*(pbA+9))  * ((int)*(pbB+9));
    *(phD+5) = ((int)*(pbA+11)) * ((int)*(pbB+11));
    *(phD+6) = ((int)*(pbA+13)) * ((int)*(pbB+13));
    *(phD+7) = ((int)*(pbA+15)) * ((int)*(pbB+15));
}                                   


//______________________________________________________



//
//  Packed unsigned Multiplication 32bit Even = 8x16bit
//
void vmp_pmullueHGeneric( uint32 * const pwD, const uint16 * const phA, const uint16 * const phB )
{
    ASSERT_PTR4(pwD);
    ASSERT_PTR4(phA);
    ASSERT_PTR4(phB);

    *(pwD+0) = ((uint)*(phA+0)) * ((uint)*(phB+0));
    *(pwD+1) = ((uint)*(phA+2)) * ((uint)*(phB+2));
    *(pwD+2) = ((uint)*(phA+4)) * ((uint)*(phB+4));
    *(pwD+3) = ((uint)*(phA+6)) * ((uint)*(phB+6));
}                                   



//
//  Packed unsigned Multiplication 32bit Odd = 8x16bit
//
void vmp_pmulluoHGeneric( uint32 * const pwD, const uint16 * const phA, const uint16 * const phB )
{
    ASSERT_PTR4(pwD);
    ASSERT_PTR4(phA);
    ASSERT_PTR4(phB);

    *(pwD+0) = ((uint)*(phA+1)) * ((uint)*(phB+1));
    *(pwD+1) = ((uint)*(phA+3)) * ((uint)*(phB+3));
    *(pwD+2) = ((uint)*(phA+5)) * ((uint)*(phB+5));
    *(pwD+3) = ((uint)*(phA+7)) * ((uint)*(phB+7));
}                                   



//
//  Packed signed Multiplication 32bit Even = 8x16bit
//
void vmp_pmulleHGeneric( int32 * const pwD, const int16 * const phA, const int16 * const phB )
{
    ASSERT_PTR4(pwD);
    ASSERT_PTR4(phA);
    ASSERT_PTR4(phB);

    *(pwD+0) = ((int)*(phA+0)) * ((int)*(phB+0));
    *(pwD+1) = ((int)*(phA+2)) * ((int)*(phB+2));
    *(pwD+2) = ((int)*(phA+4)) * ((int)*(phB+4));
    *(pwD+3) = ((int)*(phA+6)) * ((int)*(phB+6));
}                                   



//
//  Packed signed Multiplication 32bit Odd = 8x16bit
//
void vmp_pmulloHGeneric( int32 * const pwD, const int16 * const phA, const int16 * const phB )
{
    ASSERT_PTR4(pwD);
    ASSERT_PTR4(phA);
    ASSERT_PTR4(phB);

    *(pwD+0) = ((int)*(phA+1)) * ((int)*(phB+1));
    *(pwD+1) = ((int)*(phA+3)) * ((int)*(phB+3));
    *(pwD+2) = ((int)*(phA+5)) * ((int)*(phB+5));
    *(pwD+3) = ((int)*(phA+7)) * ((int)*(phB+7));
}                                   


