
//
//  PmdEx.h - Packed Multiplication/Division
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//      This module is designed for fast 3D math
//
//  Created - 31 Dec, 2001 - James Leiterman
//
//  Modifications
//
//
//  stab = 4
//


#include    "CpuAsm.h"
#include    "KariType.h"                                // Master Definitions


#ifndef __PmdEx_H
#define __PmdEx_H

#ifdef __cplusplus
extern "C" {
#endif


void PmdSetup( const CpuInfo * const pcinfo );



    //  Multiplication Highs & lows

void vmp_pmulhBGeneric(    int8 * const pbD,  const  int8  * const pbA, const  int8 * const pbB );
void vmp_pmulhuBGeneric(  uint8 * const pbD,  const uint8  * const pbA, const uint8 * const pbB );
void vmp_pmulluBGeneric(  uint8 * const pbD,  const uint8  * const pbA, const uint8 * const pbB );
#define vmp_pmullBGeneric(d,a,b) vmp_pmulluBGeneric((uint8*)d, (uint8*)a, (uint8*)b)

void vmp_pmulhHGeneric(    int16 * const phD, const  int16 * const phA, const  int16 * const phB );
void vmp_pmulhHAsmMMX(     int16 * const phD, const  int16 * const phA, const  int16 * const phB );
void vmp_pmulhHAsmSSE2(    int16 * const phD, const  int16 * const phA, const  int16 * const phB );
void vmp_pmulhHAsmSSE20(   int16 * const phD, const  int16 * const phA, const  int16 * const phB );

void vmp_pmulhuHGeneric(  uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_pmulhuHAsmMMX2(  uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_pmulhuHAsmSSE(   uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_pmulhuHAsmSSE2(  uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_pmulhuHAsmSSE20( uint16 * const phD, const uint16 * const phA, const uint16 * const phB );

void vmp_pmulluHGeneric(  uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_pmulluHAsmMMX(   uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_pmulluHAsmSSE2(  uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
void vmp_pmulluHAsmSSE20( uint16 * const phD, const uint16 * const phA, const uint16 * const phB );
#define vmp_pmullHGeneric(d,a,b) vmp_pmulluHGeneric((uint16*)d, (uint16*)a, (uint16*)b)

void vmp_pmulluWGeneric( uint32 * const pwD, const uint32 * const pwA, const uint32 * const pwB );
#define vmp_pmullWGeneric(d,a,b) vmp_pmulluWGeneric((uint32*)d, (uint32*)a, (uint32*)b)

    //  Multiplication Hi/Lo

void vmp_pmulhlBGeneric(   int8  * const pbHi,  int8  * const pbLo, const  int8  * const pbA, const  int8  * const pbB );
void vmp_pmulhluBGeneric( uint8  * const pbHi, uint8  * const pbLo, const uint8  * const pbA, const uint8  * const pbB );

void vmp_pmulhlHGeneric(   int16 * const phHi,  int16 * const phLo, const  int16 * const phA, const  int16 * const phB );
void vmp_pmulhlHAsmMMX(    int16 * const phHi,  int16 * const phLo, const  int16 * const phA, const  int16 * const phB );
void vmp_pmulhlHAsmSSE2(   int16 * const phHi,  int16 * const phLo, const  int16 * const phA, const  int16 * const phB );
void vmp_pmulhlHAsmSSE20(  int16 * const phHi,  int16 * const phLo, const  int16 * const phA, const  int16 * const phB );

void vmp_pmulhluHGeneric(  uint16 * const phHi, uint16 * const phLo, const uint16 * const phA, const uint16 * const phB );
void vmp_pmulhluHAsmMMX2(  uint16 * const phHi, uint16 * const phLo, const uint16 * const phA, const uint16 * const phB );
void vmp_pmulhluHAsmSSE(   uint16 * const phHi, uint16 * const phLo, const uint16 * const phA, const uint16 * const phB );
void vmp_pmulhluHAsmSSE2(  uint16 * const phHi, uint16 * const phLo, const uint16 * const phA, const uint16 * const phB );
void vmp_pmulhluHAsmSSE20( uint16 * const phHi, uint16 * const phLo, const uint16 * const phA, const uint16 * const phB );

    //  Multiplication Odd/Even

void vmp_pmullueBGeneric( uint16 * const phD, const uint8  * const pbA, const uint8  * const pbB );
void vmp_pmulluoBGeneric( uint16 * const phD, const uint8  * const pbA, const uint8  * const pbB );
void vmp_pmulleBGeneric(   int16 * const phD, const  int8  * const pbA, const  int8  * const pbB );
void vmp_pmulloBGeneric(   int16 * const phD, const  int8  * const pbA, const  int8  * const pbB );

void vmp_pmullueHGeneric(  uint32 * const pwD, const uint16 * const phA, const uint16 * const phB );
void vmp_pmullueHAsmMMX2(  uint32 * const pwD, const uint16 * const phA, const uint16 * const phB );
void vmp_pmullueHAsmSSE(   uint32 * const pwD, const uint16 * const phA, const uint16 * const phB );
void vmp_pmullueHAsmSSE2(  uint32 * const pwD, const uint16 * const phA, const uint16 * const phB );
void vmp_pmullueHAsmSSE20( uint32 * const pwD, const uint16 * const phA, const uint16 * const phB );

void vmp_pmulluoHGeneric(  uint32 * const pwD, const uint16 * const phA, const uint16 * const phB );
void vmp_pmulluoHAsmMMX2(  uint32 * const pwD, const uint16 * const phA, const uint16 * const phB );
void vmp_pmulluoHAsmSSE(   uint32 * const pwD, const uint16 * const phA, const uint16 * const phB );
void vmp_pmulluoHAsmSSE2(  uint32 * const pwD, const uint16 * const phA, const uint16 * const phB );
void vmp_pmulluoHAsmSSE20( uint32 * const pwD, const uint16 * const phA, const uint16 * const phB );

void vmp_pmulleHGeneric(   int32 * const pwD, const  int16 * const phA, const  int16 * const phB );
void vmp_pmulleHAsmMMX(    int32 * const pwD, const  int16 * const phA, const  int16 * const phB );
void vmp_pmulleHAsmSSE2(   int32 * const pwD, const  int16 * const phA, const  int16 * const phB );
void vmp_pmulleHAsmSSE20(  int32 * const pwD, const  int16 * const phA, const  int16 * const phB );

void vmp_pmulloHGeneric(   int32 * const pwD, const  int16 * const phA, const  int16 * const phB );
void vmp_pmulloHAsmMMX(    int32 * const pwD, const  int16 * const phA, const  int16 * const phB );
void vmp_pmulloHAsmSSE2(   int32 * const pwD, const  int16 * const phA, const  int16 * const phB );
void vmp_pmulloHAsmSSE20(  int32 * const pwD, const  int16 * const phA, const  int16 * const phB );



#ifdef __cplusplus
}
#endif

#endif  //__PmdEx_H





