
//
//  PmdX86.cpp - Packed Multiplication/Division
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 31 Dec 2001 - James Leiterman
//
//  Last Modification
//
//
//  stab=4
//



#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>

    //  Book Development Library Modules

#include "CpuAsm.h"    
#include "vmp3D.h"
#include "vmp3DEx.h"
#include "Pmd.h"
#include "PmdEx.h"



//
//  Setup Packed Multiplication/Division
//
//  NOTE: This function sets up the function vectors so the best function
//      is attached to the vector for the processor running.
//

void PmdSetup( const CpuInfo * const pcinfo )
{
    ASSERT_PTR4(pcinfo);
        
    if ( CPUBITS_SSE2 & pcinfo->nBits )
      {
        vmp_pmulhH =            vmp_pmulhHAsmSSE2;
        vmp_pmulhH0 =           vmp_pmulhHAsmSSE20;
        vmp_pmulhuH =           vmp_pmulhuHAsmSSE2;
        vmp_pmulhuH0 =          vmp_pmulhuHAsmSSE20;
        vmp_pmulluH =           vmp_pmulluHAsmSSE2;
        vmp_pmulluH0 =          vmp_pmulluHAsmSSE20;

    //  Multiplication Hi/Lo

        vmp_pmulhlH =           vmp_pmulhlHAsmSSE2;
        vmp_pmulhlH0 =          vmp_pmulhlHAsmSSE20;
        vmp_pmulhluH =          vmp_pmulhluHAsmSSE2;
        vmp_pmulhluH0 =         vmp_pmulhluHAsmSSE20;

    //  Multiplication Odd/Even

        vmp_pmullueH =          vmp_pmullueHAsmSSE2;
        vmp_pmullueH0 =         vmp_pmullueHAsmSSE20;
        vmp_pmulluoH =          vmp_pmulluoHAsmSSE2;
        vmp_pmulluoH0 =         vmp_pmulluoHAsmSSE20;
        vmp_pmulleH =           vmp_pmulleHAsmSSE2;
        vmp_pmulleH0 =          vmp_pmulleHAsmSSE20;
        vmp_pmulloH =           vmp_pmulloHAsmSSE2;
        vmp_pmulloH0 =          vmp_pmulloHAsmSSE20;
      }

    else if ( CPUBITS_SSE & pcinfo->nBits )
      {
        vmp_pmulhH =            vmp_pmulhHAsmMMX;
        vmp_pmulhH0 =           vmp_pmulhHAsmMMX;
        vmp_pmulhuH =           vmp_pmulhuHAsmSSE;
        vmp_pmulhuH0 =          vmp_pmulhuHAsmSSE;
        vmp_pmulluH =           vmp_pmulluHAsmMMX;
        vmp_pmulluH0 =          vmp_pmulluHAsmMMX;

    //  Multiplication Hi/Lo

        vmp_pmulhlH =           vmp_pmulhlHAsmMMX;
        vmp_pmulhlH0 =          vmp_pmulhlHAsmMMX;
        vmp_pmulhluH =          vmp_pmulhluHAsmSSE;
        vmp_pmulhluH0 =         vmp_pmulhluHAsmSSE;

    //  Multiplication Odd/Even

        vmp_pmullueH =          vmp_pmullueHAsmSSE;
        vmp_pmullueH0 =         vmp_pmullueHAsmSSE;
        vmp_pmulluoH =          vmp_pmulluoHAsmSSE;
        vmp_pmulluoH0 =         vmp_pmulluoHAsmSSE;
        vmp_pmulleH =           vmp_pmulleHAsmMMX;
        vmp_pmulleH0 =          vmp_pmulleHAsmMMX;
        vmp_pmulloH =           vmp_pmulloHAsmMMX;
        vmp_pmulloH0 =          vmp_pmulloHAsmMMX;
      }

    else if ( CPUBITS_3DNOW_MMX & pcinfo->nBits )
      {
        vmp_pmulhH =            vmp_pmulhHAsmMMX;
        vmp_pmulhH0 =           vmp_pmulhHAsmMMX;
        vmp_pmulhuH =           vmp_pmulhuHAsmMMX2;
        vmp_pmulhuH0 =          vmp_pmulhuHAsmMMX2;
        vmp_pmulluH =           vmp_pmulluHAsmMMX;
        vmp_pmulluH0 =          vmp_pmulluHAsmMMX;

    //  Multiplication Hi/Lo

        vmp_pmulhlH =           vmp_pmulhlHAsmMMX;
        vmp_pmulhlH0 =          vmp_pmulhlHAsmMMX;
        vmp_pmulhluH =          vmp_pmulhluHAsmMMX2;
        vmp_pmulhluH0 =         vmp_pmulhluHAsmMMX2;

    //  Multiplication Odd/Even

        vmp_pmullueH =          vmp_pmullueHAsmMMX2;
        vmp_pmullueH0 =         vmp_pmullueHAsmMMX2;
        vmp_pmulluoH =          vmp_pmulluoHAsmMMX2;
        vmp_pmulluoH0 =         vmp_pmulluoHAsmMMX2;
        vmp_pmulleH =           vmp_pmulleHAsmMMX;
        vmp_pmulleH0 =          vmp_pmulleHAsmMMX;
        vmp_pmulloH =           vmp_pmulloHAsmMMX;
        vmp_pmulloH0 =          vmp_pmulloHAsmMMX;
      }

    else if ( CPUBITS_MMX & pcinfo->nBits )
      {
        vmp_pmulhH =            vmp_pmulhHAsmMMX;
        vmp_pmulhH0 =           vmp_pmulhHAsmMMX;
        vmp_pmulhuH =           vmp_pmulhuHGeneric;
        vmp_pmulhuH0 =          vmp_pmulhuHGeneric;
        vmp_pmulluH =           vmp_pmulluHAsmMMX;
        vmp_pmulluH0 =          vmp_pmulluHAsmMMX;

    //  Multiplication Hi/Lo

        vmp_pmulhlH =           vmp_pmulhlHAsmMMX;
        vmp_pmulhlH0 =          vmp_pmulhlHAsmMMX;
        vmp_pmulhluH =          vmp_pmulhluHGeneric;
        vmp_pmulhluH0 =         vmp_pmulhluHGeneric;

    //  Multiplication Odd/Even

        vmp_pmullueH =          vmp_pmullueHGeneric;
        vmp_pmullueH0 =         vmp_pmullueHGeneric;
        vmp_pmulluoH =          vmp_pmulluoHGeneric;
        vmp_pmulluoH0 =         vmp_pmulluoHGeneric;
        vmp_pmulleH =           vmp_pmulleHAsmMMX;
        vmp_pmulleH0 =          vmp_pmulleHAsmMMX;
        vmp_pmulloH =           vmp_pmulloHAsmMMX;
        vmp_pmulloH0 =          vmp_pmulloHAsmMMX;
      }

    else
      {
        vmp_pmulhH =            vmp_pmulhHGeneric;
        vmp_pmulhH0 =           vmp_pmulhHGeneric;
        vmp_pmulhuH =           vmp_pmulhuHGeneric;
        vmp_pmulhuH0 =          vmp_pmulhuHGeneric;
        vmp_pmulluH =           vmp_pmulluHGeneric;
        vmp_pmulluH0 =          vmp_pmulluHGeneric;

    //  Multiplication Hi/Lo

        vmp_pmulhlH =           vmp_pmulhlHGeneric;
        vmp_pmulhlH0 =          vmp_pmulhlHGeneric;
        vmp_pmulhluH =          vmp_pmulhluHGeneric;
        vmp_pmulhluH0 =         vmp_pmulhluHGeneric;

    //  Multiplication Odd/Even

        vmp_pmullueH =          vmp_pmullueHGeneric;
        vmp_pmullueH0 =         vmp_pmullueHGeneric;
        vmp_pmulluoH =          vmp_pmulluoHGeneric;
        vmp_pmulluoH0 =         vmp_pmulluoHGeneric;
        vmp_pmulleH =           vmp_pmulleHGeneric;
        vmp_pmulleH0 =          vmp_pmulleHGeneric;
        vmp_pmulloH =           vmp_pmulloHGeneric;
        vmp_pmulloH0 =          vmp_pmulloHGeneric;
      }

    vmp_pmulhB =            vmp_pmulhBGeneric;
    vmp_pmulhB0 =           vmp_pmulhBGeneric;
    vmp_pmulhuB =           vmp_pmulhuBGeneric;
    vmp_pmulhuB0 =          vmp_pmulhuBGeneric;
    vmp_pmulluB =           vmp_pmulluBGeneric;
    vmp_pmulluB0 =          vmp_pmulluBGeneric;

    vmp_pmulluW =           vmp_pmulluWGeneric;
    vmp_pmulluW0 =          vmp_pmulluWGeneric;

    //  Multiplication Hi/Lo

    vmp_pmulhlB =           vmp_pmulhlBGeneric;
    vmp_pmulhlB0 =          vmp_pmulhlBGeneric;
    vmp_pmulhluB =          vmp_pmulhluBGeneric;
    vmp_pmulhluB0 =         vmp_pmulhluBGeneric;

    //  Multiplication Odd/Even

    vmp_pmullueB =          vmp_pmullueBGeneric;
    vmp_pmullueB0 =         vmp_pmullueBGeneric;
    vmp_pmulluoB =          vmp_pmulluoBGeneric;
    vmp_pmulluoB0 =         vmp_pmulluoBGeneric;
    vmp_pmulleB =           vmp_pmulleBGeneric;
    vmp_pmulleB0 =          vmp_pmulleBGeneric;
    vmp_pmulloB =           vmp_pmulloBGeneric;
    vmp_pmulloB0 =          vmp_pmulloBGeneric;

}




