        TITLE   VTrig3DX86M.asm - 80(x)86 3D Math Trignometric Function
        PAGE    53,160

;  VTrig3DX86.asm - 3D Trignometric Math
;
;  This document contains file that you may use at your own risk in your own programs.
;
;  This module is designed to accomodate the book.
;
;               This module is designed for fast 3D math
;
;  Created - 3 Oct 2001 - James Leiterman
;
;  Last Modification
;
;       Tabs = 8



        .XLIST

        .686
        .XMM

        .model flat, C

 

        include KariType.inc
        include vmp3DX86M.inc

        .LIST


OFLOW_TEST      equ     1       ; Overflow testing & fixup




        .data
        align   16

if 0
MAX_RADIAN   =     046c90fdah           ;25735.927018 radians = 1474560 degrees = 4096 circles


;himsk32        OWORD   0FFFFFFFF000000000000000000000000h  ; FFFFFFFF 00000000 00000000 00000000 
;lomsk96        OWORD   000000000FFFFFFFFFFFFFFFFFFFFFFFFh      ; 00000000 FFFFFFFF FFFFFFFF FFFFFFFF
himsk32 DWORD   000000000h,000000000h,000000000h,0FFFFFFFFh
lomsk96 DWORD   0FFFFFFFFh,0FFFFFFFFh,0FFFFFFFFh,000000000h


pio4ht      dq      0b97daa22bf490000h      ; -0.000241913 | -0.785156
mo56_42     dq      0bcc30c31bc924925h      ; -0.0238095 | -0.0178571
mo30_20     dq      0bd4ccccdbd088889h      ; -0.05 | -0.0333333
ones        dq      03F8000003F800000h      ; real4 1.0, 1.0
pio4s       dq      03f490fdb3f490fdbh      ; 0.785398 | 0.785398
mo12_6      dq      0be2aaaabbdaaaaabh      ; -0.166667 | -0.0833333
mo2s        dq      0bf000000bf000000h      ; -0.5 | -0.5

FABS_MASK   dd     07FFFFFFFh,0 ; Absolute Value Mask (clear sign bit)

four_PI    dd   03fa2f983h      ;REAL4    (4.0/PI)                      ;     dd      03fa2f983h              ; 1.27324f 
one         dd      03F800000h              ; 1.0f

toosmall        REAL4   1.0, 0.0, 0.0, 0.0      ; cos sin


foo     REAL4   0.0, 0.0, 0.0, 0.0      ;[a z y x]
foo2    REAL4   0.0, 0.0, 0.0, 0.0
foo3    REAL4   0.0, 0.0, 0.0, 0.0

endif

VTrig3DX86      SEGMENT USE32 PUBLIC 'CODE'

;-------------------------------------------------------------------------------------------------



;_________________________________________________________________________________________________
;=================================================================================================
;
;       Fixup Real4 (float)
;
;        eax=bad float
; NOTE: ST(0) already contains the float!!!

if OFLOW_TEST eq 1
FixupReal4 proc near
        fstp    qword ptr [esp-8-10]            ;Save Real8 (double) & pop ST(0)

                ; Build a Real10

        mov     eax,[esp-8-10+4]                ; Hi
        or      eax,7FFF0000h           ;Set Real8 (Double)
        mov     dword ptr [esp-10+6],eax        ;7FFFXXXX????????????

        mov     eax,[esp-8-10+4]                ; Real8 Hi
        mov     ecx,[esp-8-10+0]                ; Real8 Lo
        shld    eax,ecx,11
        shl     ecx,11
        mov     dword ptr [esp-10+4],eax        ;7FFFHHHHHHHH????????
        mov     dword ptr [esp-10+0],ecx        ;7FFFHHHHHHHHLLLLLLLL
        fld     tbyte ptr [esp-10+0]    ;Load Real10

; FIXME - Overflow handling!!!

        ret    ;ST(0)=value
FixupReal4 endp
endif


;_________________________________________________________________________________________________
;=================================================================================================
;       (float) Sine - Cosine
;
;void vmp_FSinCos( float * const pfSin, float * const pfCos, float fRads )
;{
;       ASSERT_PTR( pfSin );
;       ASSERT_PTR( pfCos );
;
;       *pfSin = sinf( fRads );
;       *pfCos = cosf( fRads );
;}
;_________________________________________________________________________________________________


        public  vmp_FSinCosAsmFPU
vmp_FSinCosAsmFPU       proc    near, pfSin:ptr, pfCos:ptr, fRads:REAL4

ASSERT_PTR( pfSin )
ASSERT_PTR( pfCos )

        fld     fRads

OFLOW_FIXUP( fRads )                            ; Overflow fixup (optional)

        fwait
        fsincos                                 ;ST(0)=cos ST(1)=sin

        mov     eax,pfSin
        mov     ecx,pfCos

        fwait

        fstp    (REAL4 PTR [ecx])                       ;ST(0)=cos ST(1)=sin
        fstp    (REAL4 PTR [eax])                       ;sin

        ret
vmp_FSinCosAsmFPU endp



;_________________________________________________________________________________________________
;=================================================================================================
;       (float) Sine
;
;void vmp_FSin( float * const pfSin, float fRads )
;{
;       ASSERT_PTR( pfSin );
;
;       *pfSin = sinf( fRads );
;}
;_________________________________________________________________________________________________


        public  vmp_FSinAsmFPU
vmp_FSinAsmFPU       proc    near, pfSin:ptr, fRads:REAL4

ASSERT_PTR( pfSin )

        fld     fRads

OFLOW_FIXUP( fRads )                            ; Overflow fixup (optional)

        fwait
        fsin                                    ;ST(0)=sin

        mov     eax,pfSin

        fwait

        fstp    (REAL4 PTR [eax])                       ;ST(0)=sin

        ret
vmp_FSinAsmFPU endp



;_________________________________________________________________________________________________
;=================================================================================================
;       (float) Cosine
;
;void vmp_FCos( float * const pfCos, float fRads )
;{
;       ASSERT_PTR( pfCos );
;
;       *pfCos = cosf( fRads );
;}
;_________________________________________________________________________________________________

        public  vmp_FCosAsmFPU
vmp_FCosAsmFPU       proc    near, pfCos:ptr, fRads:REAL4

ASSERT_PTR( pfCos )

        fld     fRads

OFLOW_FIXUP( fRads )                            ; Overflow fixup (optional)

        fwait
        fcos                                    ;ST(0)=cos

        mov     eax,pfCos

        fwait

        fstp    (REAL4 PTR [eax])                       ;ST(0)=cos

        ret
vmp_FCosAsmFPU endp





VTrig3DX86      ends
        end

