        TITLE   QVmd3DX86.asm - 80(x)86 3D Math Multiplication & Division
        PAGE    53,160

;  QVmd3D.asm - 3D (Quad) Math Multiplication & Division
;
;  This document contains file that you may use at your own risk in your own programs.
;
;  This module is designed to accomodate the book.
;
;               This module is designed for fast 3D math
;
;  Created - 3 Oct 2001 - James Leiterman
;
;  Last Modification
;
;       Tabs = 8



        .XLIST

;;;        .387
;;;     .586P   .586    .686   .686P
;;;      .K3D    .XMM    .MMX

        .686
        .XMM

        .model flat, C


        include KariType.inc
        include vmp3DX86M.inc
 
        .LIST


        .data
        align   16

himsk32 DWORD   000000000h,000000000h,000000000h,0FFFFFFFFh
lomsk96 DWORD   0FFFFFFFFh,0FFFFFFFFh,0FFFFFFFFh,000000000h
foo     REAL4   0.0,0.0,0.0,0.0

;------------------------------------------------------------------------------------------------


QVmd3DX86       SEGMENT USE32 PUBLIC 'CODE'

;-------------------------------------------------------------------------------------------------



;_________________________________________________________________________________________________
;=================================================================================================
;       (Quad) Vector Product (Multiplication)
;
;void vmp_QVecMul(vmp3DVector * const pvD, const vmp3DVector * const pvA, const vmp3DVector * const pvB)
;{
;       pvD->x = pvA->x * pvB->x;
;       pvD->y = pvA->y * pvB->y;
;       pvD->z = pvA->z * pvB->z;
;       pvD->w = pvA->w * pvB->w;
;
;       NOTE: vA, vB most likely will be misaligned especially if a packed structure
;       such as VMP3DVector is used so each field must be loaded individually!

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!
        align   16
        public  vmp_QVecMulAsm3DNow
vmp_QVecMulAsm3DNow       proc    near , vD:ptr, vA:ptr, vB:ptr
        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     edx,vB                          ; Vector B
        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

        movq    mm0,[edx+0]                     ;vB.xy   {By Bx}
        movq    mm1,[ecx+0]                     ;vA.xy   {Ay Ax}
        movq    mm2,[edx+8]                     ;vB.zw   {Bw Bz}
        movq    mm3,[ecx+8]                     ;vA.zw   {Aw Az}

        pfmul   mm1,mm0                         ;       {AyBy AxBx}
        pfmul   mm3,mm2                         ;       {AwBw AzBz}

        movq    [eax+0],mm1                     ;       {AyBy AxBx}
        movq    [eax+8],mm3                     ;       {AwBw AzBz}
        
;;;        femms
        ret
vmp_QVecMulAsm3DNow endp


;_________________________________________________________________________________________________


; *** SSE version *** non aligned
;
        align   16
        public  vmp_QVecMulAsmSSE0
vmp_QVecMulAsmSSE0       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     edx,vB                          ; Vector B
        mov     ecx,vA                          ; Vector A

        movups  xmm1,[edx]                      ;vB.xyzw {Bw Bz By Bx}
        movups  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}

        mov     eax,vD                          ; Vector Destination
        mulps   xmm0,xmm1                       ;        {AwBw AzBz AyBy AxBx}

        movups  [eax],xmm0                      ;        {AwBw AzBz AyBy AxBx}  
        
;;;        emms
        ret
vmp_QVecMulAsmSSE0 endp


;_________________________________________________________________________________________________

; *** SSE version *** 128bit aligned
;
        align   16
        public  vmp_QVecMulAsmSSE
vmp_QVecMulAsmSSE       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR16(vD)
ASSERT_PTR16(vA)
ASSERT_PTR16(vB)

        mov     edx,vB                          ; Vector B
        mov     ecx,vA                          ; Vector A

        movaps  xmm1,[edx]                      ;vB.xyzw {Bw Bz By Bx}
        movaps  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}

        mov     eax,vD                          ; Vector Destination
        mulps   xmm0,xmm1                       ;        {AwBw AzBz AyBy AxBx}

        movaps  [eax],xmm0                      ;        {AwBw AzBz AyBy AxBx}  
        
;;;        emms
        ret
vmp_QVecMulAsmSSE endp


;_________________________________________________________________________________________________
;=================================================================================================
;       (Quad) Scale Vector (w/Scalar)
;
;void vmp_QVecScale(vmp3DQVector * const pvD, const vmp3DQVector * const pvA, float fScalar)
;{
;       pvD->x = pvA->x * fScalar;
;       pvD->y = pvA->y * fScalar;
;       pvD->z = pvA->z * fScalar;
;       pvD->w = pvA->w * fScalar;
;
;       NOTE: vA, vB most likely will be misaligned especially if a packed structure
;       such as VMP3DVector is used so each field must be loaded individually!

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_QVecScaleAsm3DNow
vmp_QVecScaleAsm3DNow       proc    near , vD:ptr, vA:ptr, fScalar:REAL4
        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)

        movd    mm0,fScalar                     ;         {0  s}
        mov     ecx,vA                          ; Vector A
        punpckldq mm0,mm0                               ;         {s  s}

        mov     eax,vD                          ; Vector Destination

        movq    mm1,[ecx+0]                     ;vA.xy     {Ay Ax}
        movq    mm2,[ecx+8]                     ;vA.zw     {Aw Az}
                                                        
        pfmul   mm1,mm0                         ;         {Ays Axs}
        pfmul   mm2,mm0                         ;         {Aws Azs}

        movq    [eax+0],mm1                     ;         {Ays Axs}
        movq    [eax+8],mm2                     ;         {Aws Azs}
        
;;;        femms
        ret
vmp_QVecScaleAsm3DNow endp


;_________________________________________________________________________________________________


; *** SSE version *** non aligned
;
        align   16
        public  vmp_QVecScaleAsmSSE0
vmp_QVecScaleAsmSSE0       proc    near , vD:ptr, vA:ptr, fScalar:REAL4
        .XMM
;;;        emms

ASSERT_PTR(vD)
ASSERT_PTR(vA)

        mov     ecx,vA                          ; Vector A
        movss   xmm1,fScalar                    ;        {0 0 0 s}

        movups  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}
        shufps  xmm1,xmm1,00000000b             ; 0 0 0 0 {s  s  s  s}

        mov     eax,vD                          ; Vector Destination
        mulps   xmm0,xmm1                       ;        {Aws Azs Ays Axs}

        movups  [eax],xmm0                      ;        {Aws Azs Ays Axs}      
        
;;;        emms
        ret
vmp_QVecScaleAsmSSE0 endp


;_________________________________________________________________________________________________

; *** SSE version *** 128bit aligned
;
        align   16
        public  vmp_QVecScaleAsmSSE
vmp_QVecScaleAsmSSE       proc    near , vD:ptr, vA:ptr, fScalar:REAL4
        .XMM
;;;        emms
ASSERT_PTR16(vD)
ASSERT_PTR16(vA)

        mov     ecx,vA                          ; Vector A
        movss   xmm1,fScalar                    ;        {0 0 0 s}

        movaps  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}
        shufps  xmm1,xmm1,00000000b             ; 0 0 0 0 {s  s  s  s}

        mov     eax,vD                          ; Vector Destination
        mulps   xmm0,xmm1                       ;        {Aws Azs Ays Axs}

        movaps  [eax],xmm0                      ;        {Aws Azs Ays Axs}      

;;;        emms
        ret
vmp_QVecScaleAsmSSE endp



;_________________________________________________________________________________________________
;=================================================================================================
;       (Quad) Vector Product-Sum (Multiplication-Add)
;
;void vmp_QVecMAdd(vmp3DVector * const pvD, const vmp3DVector * const pvA, 
;           const vmp3DVector * const pvB, const vmp3DVector * const pvC)
;{
;       pvD->x = pvA->x * pvB->x + pvC->x;
;       pvD->y = pvA->y * pvB->y + pvC->y;
;       pvD->z = pvA->z * pvB->z + pvC->z;
;       pvD->w = pvA->w * pvB->w + pvC->w;
;
;       NOTE: vA, vB, vC most likely will be misaligned especially if a packed structure
;       such as VMP3DVector is used so each field must be loaded individually!

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!
        align   16
        public  vmp_QVecMAddAsm3DNow
vmp_QVecMAddAsm3DNow       proc    near , vD:ptr, vA:ptr, vB:ptr, vC:ptr
        .K3D
;;;        femms
ASSERT_PTR(vA)
ASSERT_PTR(vB)
ASSERT_PTR(vC)
ASSERT_PTR(vD)

        mov     ecx,vA                          ; Vector A
        mov     edx,vB                          ; Vector B
        mov     eax,vC                          ; Vector C

        movq    mm0,[edx+0]                     ;vB.xy   {By Bx}
        movq    mm1,[ecx+0]                     ;vA.xy   {Ay Ax}
        movq    mm4,[eax+0]                     ;vC.xy   {Cy Cx}
        movq    mm2,[edx+8]                     ;vB.zw   {Bw Bz}
        movq    mm3,[ecx+8]                     ;vA.zw   {Aw Az}
        movq    mm5,[eax+8]                     ;vC.zw  {Cw Cz}

        mov     eax,vD                          ; Vector Destination
        pfmul   mm1,mm0                         ;       {AyBy AxBx}
        pfmul   mm3,mm2                         ;       {AwBw AzBz}
        pfadd   mm1,mm4                         ;       {AyBy+Cy AxBx+Cx}
        pfadd   mm3,mm5                         ;       {AwBw+Cw AzBz+Cz}
        
        movq    [eax+0],mm1                     ;       {AyBy+Cy AxBx+Cx}
        movq    [eax+8],mm3                     ;       {AwBw+Cw AzBz+Cz}
        
;;;        femms
        ret
vmp_QVecMAddAsm3DNow endp


;_________________________________________________________________________________________________


; *** SSE version *** non aligned
;
        align   16
        public  vmp_QVecMAddAsmSSE0
vmp_QVecMAddAsmSSE0       proc    near , vD:ptr, vA:ptr, vB:ptr, vC:ptr
        .XMM
;;;        emms
ASSERT_PTR(vA)
ASSERT_PTR(vB)
ASSERT_PTR(vC)
ASSERT_PTR(vD)

        mov     ecx,vA                          ; Vector A
        mov     edx,vB                          ; Vector B
        mov     eax,vC                          ; Vector C

        movups  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}
        movups  xmm1,[edx]                      ;vB.xyzw {Bw Bz By Bx}
        movups  xmm2,[eax]                      ;vC.xyzw {Cw Cz Cy Cx}

        mov     eax,vD                          ; Vector Destination
        mulps   xmm0,xmm1                       ;        {AwBw AzBz AyBy AxBx}
        addps   xmm0,xmm2                       ;       {AwBw+Cw AzBz+Cz AyBy+Cy AxBx+Cx}

        movups  [eax],xmm0                      ;        {AwBw+Cw AzBz+Cz AyBy+Cy AxBx+Cx}

;;;        emms
        ret
vmp_QVecMAddAsmSSE0 endp


;_________________________________________________________________________________________________

; *** SSE version *** 128bit aligned
;
        align   16
        public  vmp_QVecMAddAsmSSE
vmp_QVecMAddAsmSSE       proc    near , vD:ptr, vA:ptr, vB:ptr, vC:ptr

        .XMM
;;;        emms
ASSERT_PTR16(vA)
ASSERT_PTR16(vB)
ASSERT_PTR16(vC)
ASSERT_PTR16(vD)

        mov     ecx,vA                          ; Vector A
        mov     edx,vB                          ; Vector B
        mov     eax,vC                          ; Vector C

        movaps  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}
        movaps  xmm1,[edx]                      ;vB.xyzw {Bw Bz By Bx}
        movaps  xmm2,[eax]                      ;vC.xyzw {Cw Cz Cy Cx}

        mov     eax,vD                          ; Vector Destination
        mulps   xmm0,xmm1                       ;        {AwBw AzBz AyBy AxBx}
        addps   xmm0,xmm2                       ;       {AwBw+Cw AzBz+Cz AyBy+Cy AxBx+Cx}

        movaps  [eax],xmm0                      ;        {AwBw+Cw AzBz+Cz AyBy+Cy AxBx+Cx}
        
;;;        emms
        ret
vmp_QVecMAddAsmSSE endp


;_________________________________________________________________________________________________
;=================================================================================================
;       (Quad) Vector Product-Dif (Multiplication-Sub)
;
;void vmp_QVecMSub(vmp3DVector * const pvD, const vmp3DVector * const pvA, 
;           const vmp3DVector * const pvB, const vmp3DVector * const pvC)
;{
;       pvD->x = pvA->x * pvB->x - pvC->x;
;       pvD->y = pvA->y * pvB->y - pvC->y;
;       pvD->z = pvA->z * pvB->z - pvC->z;
;       pvD->w = pvA->w * pvB->w - pvC->w;
;
;       NOTE: vA, vB, vC most likely will be misaligned especially if a packed structure
;       such as VMP3DVector is used so each field must be loaded individually!

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!
        align   16
        public  vmp_QVecMSubAsm3DNow
vmp_QVecMSubAsm3DNow       proc    near , vD:ptr, vA:ptr, vB:ptr, vC:ptr
        .K3D
;;;        femms
ASSERT_PTR(vA)
ASSERT_PTR(vB)
ASSERT_PTR(vC)
ASSERT_PTR(vD)

        mov     ecx,vA                          ; Vector A
        mov     edx,vB                          ; Vector B
        mov     eax,vC                          ; Vector C

        movq    mm0,[edx+0]                     ;vB.xy   {By Bx}
        movq    mm1,[ecx+0]                     ;vA.xy   {Ay Ax}
        movq    mm4,[eax+0]                     ;vC.xy   {Cy Cx}
        movq    mm2,[edx+8]                     ;vB.zw   {Bw Bz}
        movq    mm3,[ecx+8]                     ;vA.zw   {Aw Az}
        movq    mm5,[eax+8]                     ;vC.zw  {Cw Cz}

        mov     eax,vD                          ; Vector Destination
        pfmul   mm1,mm0                         ;       {AyBy AxBx}
        pfmul   mm3,mm2                         ;       {AwBw AzBz}
        pfsub   mm1,mm4                         ;       {AyBy-Cy AxBx-Cx}
        pfsub   mm3,mm5                         ;       {AwBw-Cw AzBz-Cz}
        
        movq    [eax+0],mm1                     ;       {AyBy-Cy AxBx-Cx}
        movq    [eax+8],mm3                     ;       {AwBw-Cw AzBz-Cz}
        
;;;        femms
        ret
vmp_QVecMSubAsm3DNow endp


;_________________________________________________________________________________________________


; *** SSE version *** non aligned
;
        align   16
        public  vmp_QVecMSubAsmSSE0
vmp_QVecMSubAsmSSE0       proc    near , vD:ptr, vA:ptr, vB:ptr, vC:ptr
        .XMM
;;;        emms
ASSERT_PTR(vA)
ASSERT_PTR(vB)
ASSERT_PTR(vC)
ASSERT_PTR(vD)

        mov     ecx,vA                          ; Vector A
        mov     edx,vB                          ; Vector B
        mov     eax,vC                          ; Vector C

        movups  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}
        movups  xmm1,[edx]                      ;vB.xyzw {Bw Bz By Bx}
        movups  xmm2,[eax]                      ;vC.xyzw {Cw Cz Cy Cx}

        mov     eax,vD                          ; Vector Destination
        mulps   xmm0,xmm1                       ;        {AwBw AzBz AyBy AxBx}
        subps   xmm0,xmm2                       ;       {AwBw-Cw AzBz-Cz AyBy-Cy AxBx-Cx}

        movups  [eax],xmm0                      ;        {AwBw-Cw AzBz-Cz AyBy-Cy AxBx-Cx}

;;;        emms
        ret
vmp_QVecMSubAsmSSE0 endp


;_________________________________________________________________________________________________

; *** SSE version *** 128bit aligned
;
        align   16
        public  vmp_QVecMSubAsmSSE
vmp_QVecMSubAsmSSE       proc    near , vD:ptr, vA:ptr, vB:ptr, vC:ptr

        .XMM
;;;        emms
ASSERT_PTR16(vA)
ASSERT_PTR16(vB)
ASSERT_PTR16(vC)
ASSERT_PTR16(vD)

        mov     ecx,vA                          ; Vector A
        mov     edx,vB                          ; Vector B
        mov     eax,vC                          ; Vector C

        movaps  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}
        movaps  xmm1,[edx]                      ;vB.xyzw {Bw Bz By Bx}
        movaps  xmm2,[eax]                      ;vC.xyzw {Cw Cz Cy Cx}

        mov     eax,vD                          ; Vector Destination
        mulps   xmm0,xmm1                       ;        {AwBw AzBz AyBy AxBx}
        subps   xmm0,xmm2                       ;       {AwBw-Cw AzBz-Cz AyBy-Cy AxBx-Cx}

        movaps  [eax],xmm0                      ;        {AwBw-Cw AzBz-Cz AyBy-Cy AxBx-Cx}
        
;;;        emms
        ret
vmp_QVecMSubAsmSSE endp


;_________________________________________________________________________________________________
;=================================================================================================
;       (Quad) Vector Division (w/Scalar)
;
;void vmp_QVecDiv(vmp3DQVector * const pvD, const vmp3DQVector * const pvA, float fScalar)
;{
;       pvD->x = pvA->x * fScalar;
;       pvD->y = pvA->y * fScalar;
;       pvD->z = pvA->z * fScalar;
;       pvD->w = pvA->w * fScalar;

;
;       NOTE: vA, vB most likely will be misaligned especially if a packed structure
;       such as VMP3DVector is used so each field must be loaded individually!

;_________________________________________________________________________________________________

; *** 3DNow version *** Slower 24bit precision
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_QVecDivAsm3DNow
vmp_QVecDivAsm3DNow       proc    near , vD:ptr, vA:ptr, fScalar:REAL4
        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_ZERO(fScalar)

        movd    mm2,fScalar                     ;         {0  s}
        mov     ecx,vA                          ; Vector A

        pfrcp   mm3,mm2                         ;         {1/s 1/s}  14bit

        mov     eax,vD                          ; Vector Destination
                ; start 24bit conversion
        punpckldq mm2,mm2                               ;         {s  s}                

        movq    mm0,[ecx+0]                     ;vA.xy     {Ay Ax}
        pfrcpit1        mm2,mm3                         ;         {1/s 1/s}

        movq    mm1,[ecx+8]                     ;vA.zw     {Aw  Az}
        pfrcpit2        mm2,mm3

        pfmul   mm0,mm2                         ;         {Ay*1/s Ax*1/s}
        pfmul   mm1,mm2                         ;         {Aw*1/s Az*1/s}

        movq    [eax+0],mm0                     ;         {Ay/s Ax/s}
        movq    [eax+8],mm1                     ;         {Aw/s Az/s}
        
;;;        femms
        ret
vmp_QVecDivAsm3DNow endp


;_________________________________________________________________________________________________

; *** 3DNow version *** FAST *** 14bit precision
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_QVecDivFastAsm3DNow
vmp_QVecDivFastAsm3DNow       proc    near , vD:ptr, vA:ptr, fScalar:REAL4
        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_ZERO(fScalar)

        movd    mm2,fScalar                     ;         {0  s}
        mov     ecx,vA                          ; Vector A

        pfrcp   mm2,mm2                         ;         {1/s 1/s}  14bit
        mov     eax,vD                          ; Vector Destination

        movq    mm0,[ecx+0]                     ;vA.xy     {Ay Ax}
        movq    mm1,[ecx+8]                     ;vA.zw     {Aw Az}

        pfmul   mm0,mm2                         ;         {Ay*1/s Ax*1/s}
        pfmul   mm1,mm2                         ;         {Aw*1/s Az*1/s}

        movq    [eax+0],mm0                     ;         {Ay/s Ax/s}
        movq    [eax+8],mm1                     ;         {Aw/s Az/s}
        
;;;        femms
        ret
vmp_QVecDivFastAsm3DNow endp


;_________________________________________________________________________________________________


; *** SSE version *** non aligned
;
        align   16
        public  vmp_QVecDivAsmSSE0
vmp_QVecDivAsmSSE0       proc    near , vD:ptr, vA:ptr, fScalar:REAL4
        .XMM
;;;        emms

ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_ZERO(fScalar)

        mov     ecx,vA                          ; Vector A
        movss   xmm1,fScalar                    ;        {0 0 0 s}

        movups  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}
        shufps  xmm1,xmm1,00000000b             ; 0 0 0 0 {s  s  s  s}

        mov     eax,vD                          ; Vector Destination
        divps   xmm0,xmm1                       ;        {Aw/s Az/s Ay/s Ax/s}

        movups  [eax],xmm0                      ;        {Aw/s Az/s Ay/s Ax/s}  
        
;;;        emms
        ret
vmp_QVecDivAsmSSE0 endp


;_________________________________________________________________________________________________

; *** SSE version *** 128bit aligned
;
        align   16
        public  vmp_QVecDivAsmSSE
vmp_QVecDivAsmSSE       proc    near , vD:ptr, vA:ptr, fScalar:REAL4
        .XMM
;;;        emms
ASSERT_PTR16(vD)
ASSERT_PTR16(vA)
ASSERT_ZERO(fScalar)

        mov     ecx,vA                          ; Vector A
        movss   xmm1,fScalar                    ;        {0 0 0 s}

        movaps  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}
        shufps  xmm1,xmm1,00000000b             ; 0 0 0 0 {s  s  s  s}

        mov     eax,vD                          ; Vector Destination
        divps   xmm0,xmm1                       ;        {Aw/s Az/s Ay/s Ax/s}
        movaps  [eax],xmm0                      ;        {Aw/s Az/s Ay/s Ax/s}  

;;;        emms
        ret
vmp_QVecDivAsmSSE endp

;_________________________________________________________________________________________________


; *** SSE version *** non aligned
;
        align   16
        public  vmp_QVecDivFastAsmSSE0
vmp_QVecDivFastAsmSSE0       proc    near , vD:ptr, vA:ptr, fScalar:REAL4
        .XMM
;;;        emms

ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_ZERO(fScalar)

        mov     ecx,vA                          ; Vector A
        movss   xmm1,fScalar                    ;        {0 0 0 s}

        movups  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}
        shufps  xmm1,xmm1,00000000b             ; 0 0 0 0 {s  s  s  s}

        mov     eax,vD                          ; Vector Destination

        rcpps   xmm1,xmm1                       ;        {1/s 1/s 1/s 1/s}
        mulps   xmm0,xmm1                       ;        {Aw*1/s Az*1/s Ay*1/s Ax*1/s}

        movups  [eax],xmm0                      ;        {Aw/s Az/s Ay/s Ax/s}  
        
;;;        emms
        ret
vmp_QVecDivFastAsmSSE0 endp


;_________________________________________________________________________________________________

; *** SSE version *** 128bit aligned
;
        align   16
        public  vmp_QVecDivFastAsmSSE
vmp_QVecDivFastAsmSSE       proc    near , vD:ptr, vA:ptr, fScalar:REAL4
        .XMM
;;;        emms
ASSERT_PTR16(vD)
ASSERT_PTR16(vA)
ASSERT_ZERO(fScalar)

        mov     ecx,vA                          ; Vector A
        movss   xmm1,fScalar                    ;        {0 0 0 s}

        movaps  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}
        shufps  xmm1,xmm1,00000000b             ; 0 0 0 0 {s  s  s  s}

        mov     eax,vD                          ; Vector Destination

        rcpps   xmm1,xmm1                       ;        {1/s 1/s 1/s 1/s}
        mulps   xmm0,xmm1                       ;        {Aw*1/s Az*1/s Ay*1/s Ax*1/s}

        movaps  [eax],xmm0                      ;        {Aw/s Az/s Ay/s Ax/s}  

;;;        emms
        ret
vmp_QVecDivFastAsmSSE endp

;_________________________________________________________________________________________________
;=================================================================================================
;       (Quad) Dot Product
;
;void vmp_QDotProduct(float * const pfD, const vmp3DQVector * const vA, const vmp3DQVector * const vB)
;{      
;       = vA->x * vB->x + vA->y * vB->y + vA->z * vB->z;   =AxBx+AyBy+AzBz
;
;       NOTE: vA, vB most likely will be misaligned especially if a packed structure
;       such as VMP3DQVector is used so each field must be loaded individually!
;

;_________________________________________________________________________________________________

; *** 3DNow version ***
;
; NOTE: If arguments not aligned, a misaligned memory stall of 1 clock will occur!

        align   16
        public  vmp_QDotProductAsm3DNow
vmp_QDotProductAsm3DNow       proc    near , vD:ptr, vA:ptr, vB:ptr
        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     edx,vB                          ; Vector B
        mov     ecx,vA                          ; Vector A

        movd    mm0,(vmp3DQVector PTR [edx]).z  ;vB.z   {0 Bz}
        movd    mm1,(vmp3DQVector PTR [ecx]).z  ;vA.z   {0 Az}
        movq    mm2,[edx]                       ;vB.xy   {By Bx}

        pfmul   mm0,mm1                         ;*vB.z  {00 BzAz}
        movq    mm3,[ecx]                       ;vA.x   {Ay Ax}

        mov     eax,vD                          ; Vector Destination
        pfmul   mm2,mm3                         ;       {ByAy BxAx}

        pfacc   mm2,mm2                         ;       {ByAy+BxAx   ByAy+BxAx}
 
        pfadd   mm0,mm2                         ;       {ByAy+BxAx+0 ByAy+BxAx+BzAz}

        movd    [eax],mm0                       ; Save resulting scalar
        
;;;        femms
        ret
vmp_QDotProductAsm3DNow endp


;_________________________________________________________________________________________________

; *** NON-Aligned SSE version ***

        align   16
        public  vmp_QDotProductAsmSSE0
vmp_QDotProductAsmSSE0       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     edx,vB                          ; Vector B
        mov     ecx,vA                          ; Vector A

        movups  xmm1,[edx]                      ;vB.xyzw {Bw Bz By Bx}
        movups  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}

        mov     eax,vD                          ; float D
        mulps   xmm0,xmm1                       ; {AwBw AzBz AyBy AxBx}

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0

        unpckhps        xmm0,xmm0                       ; {AwBw AwBw AzBz AzBz}
        shufps  xmm1,xmm1,11100001b             ; 3 2 0 1   {AwBw AzBz AxBx AyBy}
        addss   xmm2,xmm0                       ; {AwBw AzBz AyBy AzBz+AxBx}
        addss   xmm2,xmm1                       ; {AwBw AzBz AyBy AzBz+AxBx+AyBy}

        movss   [eax],xmm2                      ; Save Scalar  AzBz+AxBx+AyBy

;;;        emms
        ret
vmp_QDotProductAsmSSE0 endp

;_________________________________________________________________________________________________

; *** 128bit Aligned SSE version ***

        align   16
        public  vmp_QDotProductAsmSSE
vmp_QDotProductAsmSSE       proc    near , vD:ptr, vA:ptr, vB:ptr
        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR16(vA)
ASSERT_PTR16(vB)

        mov     edx,vB                          ; Vector B
        mov     ecx,vA                          ; Vector A

        movaps  xmm1,[edx]                      ;vB.xyzw {Bw Bz By Bx}
        movaps  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}

        mov     eax,vD                          ; float D
        mulps   xmm0,xmm1                       ; {AwBw AzBz AyBy AxBx}

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0

        unpckhps        xmm0,xmm0                       ; {AwBw AwBw AzBz AzBz}
        shufps  xmm1,xmm1,11100001b             ; 3 2 0 1  {AwBw AzBz AxBx AyBy}
        addss   xmm2,xmm0                       ; {AwBw AzBz AxBx AzBz+AxBx}
        addss   xmm2,xmm1                       ; {AwBw AzBz AxBx AzBz+AxBx+AyBy}

        movss   [eax],xmm2                      ; Save Scalar

;;;        emms
        ret
vmp_QDotProductAsmSSE endp


;_________________________________________________________________________________________________
;=================================================================================================

;void vmp_QCrossProduct(vmp3DQVector * pvD, const vmp3DQVector* pvA, const vmp3DQVector* pvB)
;{      
;       float x,y;
;            x = pvA->y * pvB->z - pvA->z * pvB->y;        AyBz-AzBy
;            y = pvA->z * pvB->x - pvA->x * pvB->z;        AzBx-AxBz
;       pvD->z = pvA->x * pvB->y - pvA->y * pvB->x;        AxBy-AyBx
;       pvD->x = x;
;       pvD->y = y;
;       pvD->w = 0.0;
;       return vD;
;
; WARNING: vD isn't saved until all calculations completed just incase vD is also vA or vB!
;
;       NOTE: vA, vB, vD most likely will be misaligned especially if a packed structure
;       such as VMP3DVector is used so each field must be loaded individually!
;

;_________________________________________________________________________________________________

; *** 3D Now! Version ***

        align   16
        public  vmp_QCrossProductAsm3DNow
vmp_QCrossProductAsm3DNow       proc    near , vD:ptr, vA:ptr, vB:ptr

        .K3D
;;;        femms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     edx,vB                          ; Vector B
        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

        movd    mm0,(vmp3DQVector PTR [edx]).x  ;vB.x   {0 Bx}
        movd    mm1,(vmp3DQVector PTR [edx]).y  ;vB.y   {0 By}
        movd    mm2,(vmp3DQVector PTR [edx]).z  ;vB.z   {0 Bz}

        movd    mm3,(vmp3DQVector PTR [ecx]).x  ;vA.x   {0 Ax}
        movd    mm4,(vmp3DQVector PTR [ecx]).y  ;vA.y   {0 Ay}
        movd    mm5,(vmp3DQVector PTR [ecx]).z  ;vA.z   {0 Az}

        pfmul   mm4,mm0                         ;vB.xy  {00 AyBx}
        punpckldq mm0,mm1                               ;       {By Bx}
        movd    mm1,(vmp3DQVector PTR [ecx]).y  ;vA.y   {0 Ay}
        punpckldq mm2,mm2                               ;       {Bz Bz}
        punpckldq mm3,mm1                               ;       {Ay Ax}
        punpckldq   mm5,mm5                     ;       {Az Az}

        pfmul   mm2,mm3                         ;*vA.xy {BzAy BzAx}
        movd    mm6,(vmp3DQVector PTR [edx]).y  ;vB.y   {0 By}
        movq    mm7,mm2                         ;       {BzAy BzAx}

        pfmul   mm5,mm0                         ;*vB.xy {AzBy AzBx}
        pfsub   mm2,mm5                         ;       {BzAy-AzBy  BzAx-AzBx}
        pfsub   mm5,mm7                         ;y@     {AzBy-BzAy  AzBx-BzAx}

        pfmul   mm6,mm3                         ;vA.xy  {0Ay ByAx}
        movd    (vmp3DQVector PTR [eax]).y,mm5  ;vD.y = AzBx-AxBz

        psrlq   mm2,32                          ;x@     {0 BzAy-AzBy}
        pfsub   mm6,mm4                         ;z@     {0-0 ByAx-AyBx}

        movd    (vmp3DQVector PTR [eax]).x,mm2  ;vD.x = AyBz-AzBy
        movd    (vmp3DQVector PTR [eax]).z,mm6  ;vD.z = AxBy-AyBx
        mov     (vmp3DQVector PTR [eax]).w,0    ;vD.w = 0.0
;;;        femms
        ret
vmp_QCrossProductAsm3DNow endp


;_________________________________________________________________________________________________

; *** NON-Aligned (128bit load/save) SSE version ***

        align   16
        public  vmp_QCrossProductAsmSSE0
vmp_QCrossProductAsmSSE0       proc    near , vD:ptr, vA:ptr, vB:ptr

        .XMM
;;;        emms
ASSERT_PTR(vD)
ASSERT_PTR(vA)
ASSERT_PTR(vB)

        mov     edx,vB                          ; Vector B
        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

        ;       z = AxBy-AyBx   y = AzBx-AxBz   x = AyBz-AzBy

        movups  xmm1,[edx]                      ;vB.xyzw {Bw Bz By Bx}
        movups  xmm0,[ecx]                      ;vA.xyzw {Bw Az Ay Ax}
        movaps  xmm5,xmm1

        shufps  xmm1,xmm1,11010010b             ;  3 1 0 2 {Bw By Bx Bz}
        movaps  xmm6,xmm0
        shufps  xmm0,xmm0,11001001b             ;  3 0 2 1 {Aw Ax Az Ay}
        shufps  xmm6,xmm6,11010010b             ;  3 1 0 2 {Aw Ay Ax Az}
        mulps   xmm1,xmm0

        shufps  xmm5,xmm5,11001001b             ;  3 0 2 1 {Bw Bx Bz By}
        movups  xmm2,[eax]                      ; Get 128bit Destination

        mulps   xmm5,xmm6
        subps   xmm1,xmm5                       ;        {# z y x}
        andps   xmm1,OWORD PTR lomsk96          ;        {0 z y x}
        movups  [eax],xmm1                      ;vD.0xyz {0 z y x}

;;;        emms
        ret
vmp_QCrossProductAsmSSE0 endp

;_________________________________________________________________________________________________

;
; *** 16byte Aligned SSE version ***
; NOTE: If arguments not aligned, will exception fault!
;
        align   16
        public  vmp_QCrossProductAsmSSE
vmp_QCrossProductAsmSSE       proc    near , vD:ptr, vA:ptr, vB:ptr

        .XMM
;;;        emms
ASSERT_PTR16(vD)
ASSERT_PTR16(vA)
ASSERT_PTR16(vB)

        mov     edx,vB                          ; Vector B
        mov     ecx,vA                          ; Vector A
        mov     eax,vD                          ; Vector Destination

        ;       z = AxBy-AyBx   y = AzBx-AxBz   x = AyBz-AzBy

        movaps  xmm1,[edx]                      ;vB.xyzw {Bw Bz By Bx}
        movaps  xmm0,[ecx]                      ;vA.xyzw {Aw Az Ay Ax}
        movaps  xmm5,xmm1

        shufps  xmm1,xmm1,11010010b             ;  3 1 0 2 {Bw By Bx Bz}
        movaps  xmm6,xmm0
        shufps  xmm0,xmm0,11001001b             ;  3 0 2 1 {Aw Ax Az Ay}
        shufps  xmm6,xmm6,11010010b             ;  3 1 0 2 {Aw Ay Ax Az}
        mulps   xmm1,xmm0

        shufps  xmm5,xmm5,11001001b             ;  3 0 2 1 {Bw Bx Bz By}
        movaps  xmm2,[eax]                      ; Get 128bit Destination

        mulps   xmm5,xmm6
        subps   xmm1,xmm5                       ;        {# z y x}
        andps   xmm1,OWORD PTR lomsk96          ;        {0 z y x}
        movaps  [eax],xmm1                      ;vD.0xyz {0 z y x}

;;;        emms
        ret
vmp_QCrossProductAsmSSE endp





;_________________________________________________________________________________________________




QVmd3DX86       ends
        end

