
//
//  Bench.cpp - Vector 3D Multiplication & Division
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 5 Oct 2001 - James Leiterman
//
//  Last Modification
//
//
//  stab=4
//

#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries

#include <iostream.h>
#include <memory.h>

    //  Book Development Library Modules

#include "CpuAsm.h"                                     // CPU Module
#include "vmp3D.h"
#include "Vmd3D.h"
#include "Vmd3DEx.h"



typedef enum
{
    iTEST_DOT_PRODUCT,
    iTEST_CROSS_PRODUCT,
    iTEST_VEC_MUL,
    iTEST_VEC_MADD,
    iTEST_VEC_MSUB,
    iTEST_VEC_SCALE,
    iTEST_VEC_DIV,
    iTEST_VEC_DIV_FAST
} iTEST;



//
//  Test 
//

static void testVec( bool bAlign, iTEST iFunc )
{
    vmp3DVector *pvA, *pvB, *pvC, *pvTmp, *pvTmp2;
    int x, y, z, dif;
    float f, f2, fPrecision, fScalar, fWTst;
    float buf[4*4  +3];         // enough space + 12 bytes (3 floats) extra
    bool bF;

        // Force proper alignment (Crude just for this demo)

    bF = false;

    if (bAlign)
      {
        pvA    = (vmp3DVector*) ALIGN16((int)(buf));
        pvB    = (vmp3DVector*) ALIGN16((int)(pvA+1));
        pvTmp  = (vmp3DVector*) ALIGN16((int)(pvB+1));
        pvTmp2 = (vmp3DVector*) ALIGN16((int)(pvTmp+1));
      }
    else
      {
        pvA = (vmp3DVector*) buf;
        if (!(0x000f & (int)pvA))
          {
            pvA = (vmp3DVector*) (buf+1);           // Make sure misaligned!
          }

        pvB    = pvA + 1;
        pvTmp2 = pvB + 1;
        pvTmp  = pvTmp2 + 1;
      }

    pvC = pvA;
    dif = 4;
    fPrecision = SINGLE_PRECISION;
    
    for ( z = -dif; z <= dif; z++ )
      {
        pvA->z = 0.123f *(float)z;
        pvB->y = 0.123f *(float)z;

        for ( y = -dif; y <= dif; y++ )
          {
            pvA->y = 0.123f *(float)y;
            pvB->x = 0.123f *(float)y;

            for ( x = -dif; x <= dif; x++ )
              { 
                pvA->x = 0.123f *(float)x;
                pvB->z = 0.123f *(float)x;
                fScalar = pvA->x;

                fWTst = ((vmp3DQVector *) pvTmp)->w = fScalar;  // Get untouchable value!

                switch(iFunc)
                  {
                    case iTEST_DOT_PRODUCT:
                        fPrecision = SINGLE_PRECISION;
                        bF = true;
                        vmp_DotProductGeneric( &f2, pvA, pvB );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_DotProduct( &f, pvA, pvB );     // Aligned
                        else
                            vmp_DotProduct0( &f, pvA, pvB );
                        vmp_SIMDExit();
                        break;

                    case iTEST_CROSS_PRODUCT:
                        fPrecision = SINGLE_PRECISION;
                        bF = false;
                        vmp_CrossProductGeneric( pvTmp2, pvA, pvB );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_CrossProduct( pvTmp, pvA, pvB );    // Aligned
                        else
                            vmp_CrossProduct0( pvTmp, pvA, pvB );
                        vmp_SIMDExit();
                        break;

                    case iTEST_VEC_MUL:
                        fPrecision = SINGLE_PRECISION;
                        bF = false;
                        vmp_VecMulGeneric( pvTmp2, pvA, pvB );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_VecMul( pvTmp, pvA, pvB );          // Aligned
                        else
                            vmp_VecMul0( pvTmp, pvA, pvB );
                        vmp_SIMDExit();
                        break;

                    case iTEST_VEC_MADD:
                        fPrecision = SINGLE_PRECISION;
                        bF = false;
                        vmp_VecMAddGeneric( pvTmp2, pvA, pvB, pvC );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_VecMAdd( pvTmp, pvA, pvB, pvC );            // Aligned
                        else
                            vmp_VecMAdd0( pvTmp, pvA, pvB, pvC );
                        vmp_SIMDExit();
                        break;

                    case iTEST_VEC_MSUB:
                        fPrecision = SINGLE_PRECISION;
                        bF = false;
                        vmp_VecMSubGeneric( pvTmp2, pvA, pvB, pvC );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_VecMSub( pvTmp, pvA, pvB, pvC );            // Aligned
                        else
                            vmp_VecMSub0( pvTmp, pvA, pvB, pvC );
                        vmp_SIMDExit();
                        break;

                    case iTEST_VEC_SCALE:
                        fPrecision = SINGLE_PRECISION;
                        bF = false;
                        vmp_VecScaleGeneric( pvTmp2, pvA, fScalar );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_VecScale( pvTmp, pvA, fScalar );    // Aligned
                        else
                            vmp_VecScale0( pvTmp, pvA, fScalar );
                        vmp_SIMDExit();
                        break;

                    case iTEST_VEC_DIV:
                        fPrecision = SINGLE_PRECISION;
                        bF = false;
                        if (0.0==fScalar) continue;

                        vmp_VecDivGeneric( pvTmp2, pvA, fScalar );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_VecDiv( pvTmp, pvA, fScalar );          // Aligned
                        else
                            vmp_VecDiv0( pvTmp, pvA, fScalar );
                        vmp_SIMDExit();
                        break;

                    case iTEST_VEC_DIV_FAST:
                        fPrecision = FAST_PRECISION;
                        bF = false;
                        if (0.0==fScalar) continue;

                        vmp_VecDivGeneric( pvTmp2, pvA, fScalar );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_VecDivFast( pvTmp, pvA, fScalar );          // Aligned
                        else
                            vmp_VecDivFast0( pvTmp, pvA, fScalar );
                        vmp_SIMDExit();
                        break;
                  }

                if (bF)
                  {
                    if (!vmp_IsFEqual(f, f2, fPrecision))
                      {
                        cout << " vA.x:" << pvA->x << " vA.y:" << pvA->y << " vA.z:" << pvA->z << "\n"; 
                        cout << " vB.x:" << pvB->x << " vB.y:" << pvB->y << " vB.z:" << pvB->z << "\n"; 
                        cout << " " << f2 << " Good vs Bad! " << f << "  *** MISMATCH!!! ***\n" << endl;
                      }
                  }
                else
                  {
                    if (!vmp_IsVEqual(pvTmp, pvTmp2, fPrecision)) 
                      {
                        cout << " vA.x:" << pvA->x << " vA.y:" << pvA->y << " vA.z:" << pvA->z << "\n";
                        cout << " vB.x:" << pvB->x << " vB.y:" << pvB->y << " vB.z:" << pvB->z << "\n"; 
                        cout << "    x:" << pvTmp2->x << " y:" << pvTmp2->y << " z:" << pvTmp2->z << "  Good vs Bad!!!\n";
                        cout << "    x:" << pvTmp->x << " y:" << pvTmp->y << " z:" << pvTmp->z << "   *** MISMATCH!!! ***\n" << endl;
                      }

                        //  Check for '.W' contamination
                    if (!vmp_IsFEqual(fWTst, ((vmp3DQVector *) pvTmp)->w, fPrecision ))
                      {
                        cout << ".W is contaminated! should be: " << fWTst << " not " << ((vmp3DQVector *) pvTmp)->w << endl;
                      }
                  }
              }
          }
      }
}



int main( int argc, char *argv[] )
{
    CpuInfo cinfo;
    char szBuf[ CPU_SZBUF_MAX ];

    CpuDetect( &cinfo );                                // Detect CPU

    cout << "\nVector Multiplication, Division, Scaling, Dot, and Cross Test\n\n";
    cout << CpuInfoStr( szBuf, &cinfo ) << endl;

    Vmd3DSetup( &cinfo );                               // Setup (Vector) stuff

    cout << "Testing Scaling" << endl; 
    testVec( false, iTEST_VEC_SCALE );
    
    cout << "Testing Scaling (Aligned)" << endl;
    testVec( true,  iTEST_VEC_SCALE );

    cout << "Testing Vector Multiplication" << endl;
    testVec( false, iTEST_VEC_MUL );

    cout << "Testing Vector Multiplication (Aligned)" << endl;
    testVec( true,  iTEST_VEC_MUL );

    cout << "Testing Vector Multiplication-Add" << endl;
    testVec( false, iTEST_VEC_MADD );

    cout << "Testing Vector Multiplication-Add (Aligned)" << endl;
    testVec( true,  iTEST_VEC_MADD );

    cout << "Testing Vector Multiplication-Sub" << endl;
    testVec( false, iTEST_VEC_MSUB );

    cout << "Testing Vector Multiplication-Sub (Aligned)" << endl;
    testVec( true,  iTEST_VEC_MSUB );

    cout << "Testing Vector Division" << endl;
    testVec( false, iTEST_VEC_DIV );

    cout << "Testing Vector Division (Aligned)" << endl;
    testVec( true,  iTEST_VEC_DIV );

    cout << "Testing Vector Division *14bit FAST*)" << endl;
    testVec( false, iTEST_VEC_DIV_FAST );

    cout << "Testing Vector Division (Aligned) *14bit FAST*" << endl;
    testVec( true,  iTEST_VEC_DIV_FAST );

    cout << "Testing Dot Product" << endl;
    testVec( false, iTEST_DOT_PRODUCT );

    cout << "Testing Dot Product (Aligned)" << endl;
    testVec( true,  iTEST_DOT_PRODUCT );

    cout << "Testing Cross Product" << endl;
    testVec( false, iTEST_CROSS_PRODUCT );

    cout << "Testing Cross Product (Aligned)" << endl;
    testVec( true,  iTEST_CROSS_PRODUCT );

    cout << "Fin\n\n" << flush;

    return 0;
}

