
//
//  MCompX86.cpp - Motion Compensation
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 21 Feb, 2002 - James Leiterman
//
//  Last Modification
//
//
//  stab=4
//



#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>

    //  Book Development Library Modules

#include "CpuAsm.h"    
#include "vmp3D.h"
#include "vmp3DEx.h"
#include "MComp.h"
#include "MCompEx.h"



//
//  Setup Motion Compensation
//
//  NOTE: This function sets up the function vectors so the best function
//      is attached to the vector for the processor running.
//

void MCompSetup( const CpuInfo * const pcinfo )
{
    ASSERT_PTR(pcinfo);
        
    if ( CPUBITS_SSE2 & pcinfo->nBits )
      {
        vmp_MCompHz8R =         vmp_MCompHz8RAsmSSE;
        vmp_MCompHz16R =        vmp_MCompHz16RAsmSSE2;
        vmp_MCompVt8R =         vmp_MCompVt8RAsmSSE;
        vmp_MCompVt16R =        vmp_MCompVt16RAsmSSE2;
      }

    else if ( CPUBITS_SSE & pcinfo->nBits )
      {
        vmp_MCompHz8R =         vmp_MCompHz8RAsmSSE;
        vmp_MCompHz16R =        vmp_MCompHz16RAsmSSE;
        vmp_MCompVt8R =         vmp_MCompVt8RAsmSSE;
        vmp_MCompVt16R =        vmp_MCompVt16RAsmSSE;
      }

    else if ( CPUBITS_3DNOW & pcinfo->nBits )
      {
        vmp_MCompHz8R =         vmp_MCompHz8RAsm3DNow;
        vmp_MCompHz16R =        vmp_MCompHz16RAsm3DNow;
        vmp_MCompVt8R =         vmp_MCompVt8RAsm3DNow;
        vmp_MCompVt16R =        vmp_MCompVt16RAsm3DNow;
      }

    else if ( CPUBITS_MMX & pcinfo->nBits )
      {
        vmp_MCompHz8R =         vmp_MCompHz8RAsmMMX;
        vmp_MCompHz16R =        vmp_MCompHz16RAsmMMX;
        vmp_MCompVt8R =         vmp_MCompVt8RAsmMMX;
        vmp_MCompVt16R =        vmp_MCompVt16RAsmMMX;
      }

    else
      {
        vmp_MCompHz8R =         vmp_MCompHz8RGeneric;
        vmp_MCompHz16R =        vmp_MCompHz16RGeneric;
        vmp_MCompVt8R =         vmp_MCompVt8RGeneric;
        vmp_MCompVt16R =        vmp_MCompVt16RGeneric;
      }
}




