
;
; CpuX86.inc - 80(X)86 Cpu definitions
;
; This document contains file that you may use at your own risk in your own programs.
;
; This module is designed to work with the exercises for this book.
;
;   Created - 5 Jul 98 - J.Leiterman
;
;   Modifications
;
;       23 Jul 2001 - JCL - Brought up to date.
;
;   Tabs = 8
;


        include x86.inc


ifndef __CpuX86Inc
__CpuX86Inc equ 1


;   CPU definition Id's

CPU_086         =       0       ; 8086
CPU_186         =       1       ; 80186
CPU_286         =       2       ; 80286
CPU_386         =       3       ; 80386
CPU_486         =       4       ; 80486
CPU_PENTIUM     =       5       ; P5 (Pentium)
CPU_PENTIUM_PRO =       6       ; Pentium Pro
CPU_PII         =       6       ; PII


;   FPU definition Id's
;   All Pentiums and above have a built in FPU.
;   Some 486's also had a built in FPU.  But below that 
;   there are certain matched pairs!
;       CPU_086     NONE, FPU_087
;       CPU_186     NONE, FPU_087
;       CPU_286     NONE, FPU_287
;       CPU_386     NONE, FPU_287, FPU_387
;       CPU_486     NONE, FPU_387, FPU_487

FPU_NONE        =       0       ; No FPU chip
FPU_087         =       1       ; 8087
FPU_287         =       2       ; 80287
FPU_387         =       3       ; 80387
FPU_487         =       CPU_486
FPU_PENTIUM     =       CPU_PENTIUM
FPU_PII         =       CPU_PII



;   CPU bit flags

CPUBITS_FPU         =   0000000000000001b   ; FPU flag
CPUBITS_MMX         =   0000000000000010b   ; MMX flag
CPUBITS_3DNOW       =   0000000000000100b   ; 3DNow flag
CPUBITS_FXSR        =   0000000000001000b   ; Fast FP Store
CPUBITS_SSE         =   0000000000010000b   ; Katmai NI
CPUBITS_SSE2        =   0000000000100000b
CPUBITS_3DNOW_MMX   =   0000000001000000b   ; 3DNow (MMX Extensions)
CPUBITS_3DNOW_EXT   =   0000000010000000b   ; 3DNow (Extensions)
CPUBITS_3DNOW_SSE   =   0000000100000000b   ; 3DNow! Professional
CPUBITS_HTT			=   0000001000000000b   ; Hyperthreading Tech
CPUBITS_SSE3		=   0000010000000000b   ; Prescott NI
CPUBITS_EM64T		=	0000100000000000b   ; EM64T supported
CPUBITS_AMD64		=   0001000000000000b   ; AMD Long Mode



;   CPU Vendors

CPUVEN_UNKNOWN      =   0
CPUVEN_INTEL        =   1
CPUVEN_AMD          =   2
CPUVEN_CYRIX        =   3
CPUVEN_CENTAUR      =   4
CPUVEN_NATIONAL     =   5
CPUVEN_UMC          =   6
CPUVEN_NEXGEN       =   7
CPUVEN_RISE         =   8
CPUVEN_TRANSMETA    =   9


;
;   Cpu Information
;

CpuInfo struct 4
    nCpuId      dd      0       ; Cpu type Identifier
    nFpuId      dd      0       ; Floating Point Unit Identifier
    nBits       dd      0       ; Feature Bits
    nMfg        dd      0       ; Manufacturer
	nProcCnt	db		0		; # of logical processors (this processor)
	pad			db		0,0,0
CpuInfo ends




;extern CpuDetect


endif   ; __CpuX86Inc

